#        Name : project_faces_plane
# Description : projects the selected set of faces perpendicular to the selected face
#      Author : Didier Bur
#       Usage : select faces and a face double-click and run the script, you'll get lines and/or faces which are the 
#               perpendicular projections of the selected faces on the selected face.
#        Date : 09.Sept.2oo4
#        Type : tool

require 'sketchup.rb'

def project_faces_plane

model = Sketchup.active_model
model.start_operation "Project lines on face"
entities = model.active_entities
ss = model.selection

if ss.empty? 
  UI.messagebox("No selection.")
  return nil
end

# Who are the faces and who's the face to project on ?
i = 0
plane_face = []
0.upto( ss.length - 1 ) do |element|
  element = ss[i]
  if( element.typename == "Edge")
    plane_face = element.faces
    #ss.remove(element)
  end
  i = i + 1
end # of upto

if( plane_face == nil )
  UI.messagebox("No projection face selected.")
  return nil
end

# Remove the face on which to project from selection
if( plane_face )
  ss.remove(plane_face)
end


# Face's plane equation
plan = plane_face[0].plane

# sets the default projection settings
@create_faces = "No" if not @create_faces

# Dialog box
create_faces = ["Yes","No"]
enums = [create_faces.join("|")]
prompts = ["Create faces  "]
values = [@create_faces]

results = inputbox prompts, values, enums, "Parameters"

# User cancels
return if not results
# Otherwise
@create_faces = results
 
# let's go for the projections
i = 0

0.upto( ss.length - 1) do |something|
  current_face = ss[i]
  if( current_face.typename != "Edge")
    current_edges = current_face.edges

    j = 0
    0.upto( current_edges.length - 1) do |extract_vertex|
      current_edge = current_edges[j]
      points = current_edge.vertices
      new_pt1 = points[0].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
      new_pt2 = points[1].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
      Sketchup.active_model.active_entities.add_line( new_pt1, new_pt2 )
      if(  @create_faces[0] == "Yes" )
        Sketchup.active_model.active_entities.add_face( points[0].position, points[1].position, new_pt2, new_pt1 )
      end
      #Next edge
      j = j + 1
    end #of upto
  end #of if not edge
  #Next face
  i = i + 1
end # of upto

end # of def

if( not file_loaded?("project_faces_plane.rb") )

    # This will add a separator to the menu, but only once
    add_separator_to_menu("Plugins")
    
    # To add an item to a menu, you identify the menu, and then
    # provide a title to display and a block to execute.  In this case,
    # the block just calls the create_box function
    UI.menu("Plugins").add_item("Project faces on a face") { project_faces_plane }

end

#-----------------------------------------------------------------------------
file_loaded("project_faces_plane.rb")