#        Name : project_faces_vector
# Description : projects the selected set of faces on the selected face, along a vector
#      Author : Didier Bur
#       Usage : select faces and a face double-click and run the script, you'll get lines and/or faces which are the 
#               projections of the selected faces on the selected face, along the selected vector
#        Date : 27.Sept.2oo4
#        Type : tool

require 'sketchup.rb'

def project_faces_vector

model = Sketchup.active_model
model.start_operation "Project lines on face"
entities = model.active_entities
ss = model.selection

if ss.empty? 
  UI.messagebox("No selection.")
  return nil
end

# Who are the faces and who's the face to project on ?
i = 0
plane_face = nil
v = nil

0.upto( ss.length - 1 ) do |element|
  element = ss[i]
  if( element.typename == "Edge") and (element.faces[0] != nil)
    plane_face = element.faces
    #puts plane_face.to_s + " " + i.to_s + "\n"
    else
      if( element.typename == "Edge") and (element.faces[0] == nil)
      v = element
      end
  end
  i = i + 1
end # of upto

#puts "found vector: " + v.to_s + "\n"

if( plane_face == nil )
  UI.messagebox("No projection face selected.")
  return nil
end

# Remove the face on which to project from selection, its edges and the vector

if( plane_face )
  loop = plane_face[0].loops
  ss.remove(plane_face)
  ss.remove(loop[0].edges)
  ss.remove(v)
end


# Face's plane equation and vector
plan = plane_face[0].plane
vect = Geom::Vector3d.new(v.line[1])

#puts vect.to_s + "\n"

# sets the default projection settings
@create_faces = "No" if not @create_faces

# Dialog box
create_faces = ["Yes","No"]
enums = [create_faces.join("|")]
prompts = ["Create faces  "]
values = [@create_faces]

results = inputbox prompts, values, enums, "Parameters"

# User cancels
return if not results
# Otherwise
@create_faces = results
 
# let's go for the projections
i = 0
trans = Geom::Transformation.translation(vect)

0.upto( ss.length - 1) do |something|
  current_face = ss[i]
  if( current_face.typename != "Edge")
    current_edges = current_face.edges

    j = 0
    0.upto( current_edges.length - 1) do |extract_vertex|
      current_edge = current_edges[j]
      points = current_edge.vertices
      
      new_pt1 = points[0].position.transform(trans)
      new_line = [points[0].position, new_pt1]
      new_pt1bis = Geom.intersect_line_plane(new_line, plan)
      new_pt2 = points[1].position.transform(trans)
      new_line = [points[1].position, new_pt2]
      new_pt2bis = Geom.intersect_line_plane(new_line, plan)
      
      #Sketchup.active_model.active_entities.add_cpoint new_pt1bis
      #Sketchup.active_model.active_entities.add_cpoint new_pt2bis
      
      #new_pt1 = points[0].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
      #new_pt2 = points[1].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
      Sketchup.active_model.active_entities.add_line( new_pt1bis, new_pt2bis )
      if(  @create_faces[0] == "Yes" )
        Sketchup.active_model.active_entities.add_face( points[0].position, points[1].position, new_pt2bis, new_pt1bis )
      end
      #Next edge
      j = j + 1
    end #of upto
  end #of if not edge
  #Next face
  i = i + 1
end # of upto

end # of def

if( not file_loaded?("project_faces_vector.rb") )
    add_separator_to_menu("Plugins")
    UI.menu("Plugins").add_item("Project faces on a plane along vector") { project_faces_vector }

end

#-----------------------------------------------------------------------------
file_loaded("project_faces_vector.rb")