#        Name : project_line_plane
# Description : projects the selected construction point perpendicular to the selected face
#      Author : Didier Bur
#       Usage : select a cpoint and a face and run the script, you'll get another cpoint on the line which is the perpendicular projection of the selected cpoint on that face.
#        Date : 11.Aug.2oo4
#        Type : tool

require 'sketchup.rb'

def project_line_plane

model = Sketchup.active_model
model.start_operation "Project line on face"
entities = model.active_entities
ss = model.selection

if ss.empty? 
  UI.messagebox("No selection.")
  return nil
end

if( ss.length > 2 )
  UI.messagebox("Please select only a line and a face.")
  return nil
end

# Who's the line and who's the face ?
ent1 = ss[0]
ent2 = ss[1]

if( ent1.typename == "Face")
  fc = ent1
  ln = ent2
  else
  ln = ent1
  fc = ent2
end

# sets the default projection settings
@create_face = "No" if not @create_face

# Dialog box
create_face = ["Yes","No"]
enums = [create_face.join("|")]
prompts = ["Create a face  "]
values = [@create_face]

results = inputbox prompts, values, enums, "Parameters"

# User cancels
return if not results
# Otherwise
@create_face = results
 
# let's go for the projection 
vec = fc.normal.normalize!      # not useful...
points = []
points = ln.vertices
plan = fc.plane

new_pt1 = points[0].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
new_pt2 = points[1].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
Sketchup.active_model.active_entities.add_line( new_pt1, new_pt2 )

if(  @create_face[0] == "Yes" )
  Sketchup.active_model.active_entities.add_face( points[0].position, points[1].position, new_pt2, new_pt1 )
end

end # of def

if( not file_loaded?("project_line_plane.rb") )

    # This will add a separator to the menu, but only once
    add_separator_to_menu("Plugins")
    
    # To add an item to a menu, you identify the menu, and then
    # provide a title to display and a block to execute.  In this case,
    # the block just calls the create_box function
    UI.menu("Plugins").add_item("Project a line on a face") { project_line_plane }

end

#-----------------------------------------------------------------------------
file_loaded("project_line_plane.rb")