#        Name : project_lines_plane
# Description : projects the selected set of lines perpendicular to the selected face
#      Author : Didier Bur
#       Usage : select lines and a face and run the script, you'll get lines and/or faces which are the 
#               perpendicular projections of the selected lines on the selected face.
#        Date : 12.Aug.2oo4
#        Type : tool

require 'sketchup.rb'

def project_lines_plane

model = Sketchup.active_model
model.start_operation "Project lines on face"
entities = model.active_entities
ss = model.selection

if ss.empty? 
  UI.messagebox("No selection.")
  return nil
end

# Who are the lines and who's the face ?
i = 0
indice_element = 0
face_selected = 0
0.upto( ss.length - 1 ) do |something|
  element = ss[i]
  if( element.typename == "Face")
    indice_element = i
    face_selected = 1
  end
  i = i + 1
end # of upto

if( face_selected == 0 )
  UI.messagebox("Please select a face too.")
  return nil
end

# Face's plane equation
plan = ss[indice_element].plane
# remove the face from the selection set
ss.remove(ss[indice_element])

# sets the default projection settings
@create_faces = "No" if not @create_faces

# Dialog box
create_faces = ["Yes","No"]
enums = [create_faces.join("|")]
prompts = ["Create faces  "]
values = [@create_faces]

results = inputbox prompts, values, enums, "Parameters"

# User cancels
return if not results
# Otherwise
@create_faces = results
 
# let's go for the projections
i = 0

0.upto( ss.length - 1 ) do |something|
  ln = ss[i]
  points = []
  points = ln.vertices
  new_pt1 = points[0].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
  new_pt2 = points[1].position.project_to_plane(plan[0], plan[1], plan[2], plan[3])
  Sketchup.active_model.active_entities.add_line( new_pt1, new_pt2 )

  if(  @create_faces[0] == "Yes" )
    Sketchup.active_model.active_entities.add_face( points[0].position, points[1].position, new_pt2, new_pt1 )
  end

  i = i + 1
end # of upto

end # of def

if( not file_loaded?("project_lines_plane.rb") )

    # This will add a separator to the menu, but only once
    add_separator_to_menu("Plugins")
    
    # To add an item to a menu, you identify the menu, and then
    # provide a title to display and a block to execute.  In this case,
    # the block just calls the create_box function
    UI.menu("Plugins").add_item("Project lines on a face") { project_lines_plane }

end

#-----------------------------------------------------------------------------
file_loaded("project_lines_plane.rb")