# SketchUp to DXF or STL Converter (mesh) 
# Last edited: 04 July 2007
# Author: Nathan Bromham, based on the KML exporter of Michael Ashbridge   

require 'sketchup.rb'

def export_mesh_file ( file_type )
  if (file_type == "dxf") || (file_type == "stl")
	 model = Sketchup.active_model
	 ss = model.selection
	 entities = model.entities
	 faces_array = []
	 model.start_operation "export_dxf_mesh"
	 if ss.empty? 
	   	answer = UI.messagebox("No objects selected. Export entire model?", MB_YESNOCANCEL)
		  else
		  export_ents = Sketchup.active_model.selection
	 end
	 if( answer == 6 )
	   	export_ents = model.entities
	 end
	 if( answer == 6 ) or (not ss.empty?)
	 (erase_mesh)
	 out_name = UI.openpanel( file_type+" file location", "." , "#{File.basename(model.path).split(".")[0]}." +file_type )
	 mesh_file = File.new( out_name , "w" )
	 others = 0
	 i = 0
	 0.upto(export_ents.length - 1) do |e|
	 	element = export_ents[i]
		if( element.typename != "Face")
			others = others + 1
		else
		faces_array.push element
		end
		i = i + 1
	 end
	 model.layers.add("MESH_data")
	 model.active_layer = ("MESH_data")
    model.layers[0].visible=false
	 model.layers[1].visible=true
	 faces_array.each do |face|
	   	mesh=face.mesh(7)
		  normal = mesh.normal_at 1
		  entities.add_faces_from_mesh mesh, 9
	 end
	 (mesh_bucket)
	 if (file_type == "dxf")
	   (export_mesh mesh_file)
	 else
	   (export_mesh_stl mesh_file)
	 end
	 UI.messagebox( faces_array.length.to_s + " faces converted to " + $triangles_array.length.to_s + " triangles \n" + others.to_s + " objects ignored" )
	 $triangles_array = []
	 end
	 model.commit_operation
	end
end

def mesh_bucket
  model = Sketchup.active_model
  $triangles_array = []
  entities = model.entities
  i = 0
  entities.each do |f|
    element = entities[i]
    if(element.typename == "Face" and element.layer.name == "MESH_data")
      $triangles_array.push element
    end
    i = i + 1
  end
end

def erase_mesh
  model = Sketchup.active_model
  ent_arr=[]
  model.entities.each do |element|
    if(element.layer.name == "MESH_data"  )
    ent_arr.push element
    end
  end
  ent_arr.each {|e| e.erase! if not e.deleted?}
end

def export_mesh( dxf_file )
  model = Sketchup.active_model
  model_filename = File.basename(model.path)
  if( model_filename == "" )
    model_filename = "no_name"
  end
  model_name = model_filename.split(".")[0]
  dxf_file.puts( " 0")
  dxf_file.puts( "SECTION")
	dxf_file.puts( " 2")
	dxf_file.puts( "ENTITIES")
	i = 0
	$triangles_array.each do |tri|
		vertices = tri.vertices
    dxf_file.puts( "  0")
    dxf_file.puts( "3DFACE")
    dxf_file.puts( "  8")
    dxf_file.puts( "MY3DLAYER")
    for j in 0..3 do
      if (j==3) 
        count = 2
      else
        count = j
      end
      dxf_file.puts( 10+j)#x
      dxf_file.puts(vertices[count].position.x.to_f.to_s)
      dxf_file.puts( 20+j)#y
      dxf_file.puts(vertices[count].position.y.to_f.to_s)
      dxf_file.puts( 30+j)#z  
      dxf_file.puts(vertices[count].position.z.to_f.to_s)
    end
		i = i + 1
		done = ((i * 100.0) / $triangles_array.length).to_i
		Sketchup.set_status_text("Exporting mesh: " + done.to_s + "%")
	end
	dxf_file.puts( "  0")
	dxf_file.puts( "ENDSEC")
	dxf_file.puts( "  0")
	dxf_file.puts( "EOF")
	dxf_file.close
end

def export_mesh_stl( stl_file )
  model = Sketchup.active_model
  model_filename = File.basename(model.path)
  if( model_filename == "" )
    model_filename = "no_name"
  end
  model_name = model_filename.split(".")[0]
  stl_file.puts( "solid " + model_filename)
	i = 0
	$triangles_array.each do |tri|
		vertices = tri.vertices
    stl_file.puts( "facet normal " + tri.normal.x.to_s + " " + tri.normal.y.to_s + " " + tri.normal.z.to_s)
    stl_file.puts( "outer loop")
    for j in 0..2 do
      stl_file.puts("vertex " + vertices[j].position.x.to_f.to_s + " " + vertices[j].position.y.to_f.to_s + " " + vertices[j].position.z.to_f.to_s)
    end
    stl_file.puts( "endloop")
    stl_file.puts( "endfacet")
		i = i + 1
		done = ((i * 100.0) / $triangles_array.length).to_i
		Sketchup.set_status_text("Exporting mesh: " + done.to_s + "%")
	end
	stl_file.puts( "endsolid " + model_filename)
	stl_file.close
end

if( not file_loaded?("sketchup_to_dxf_stl.rb") )
  #add_separator_to_menu("Plugins")
  UI.menu("Plugins").add_item("Export DXF (mesh)") { export_mesh_file ( "dxf" ) }
  UI.menu("Plugins").add_item("Export STL (mesh)") { export_mesh_file ( "stl" ) }
end
file_loaded("to_dxf_stl_mesh.rb")
