# Copyright 2008, LEDAS

# NO PART OF THIS SOFTWARE MAY BE RE-USED FOR ANY OTHER PURPOSE THAN TO RUN
# DRIVING DIMENSIONS WITHOUT THE EXPRESS AND WRITTEN APPROVAL OF LEDAS.


# YOU UNDERSTAND AND AGREE THAT LEDAS IS MAKING NO WARRANTY THAT THE SOFTWARE WILL
# MEET YOUR REQUIREMENTS, THAT THE SOFTWARE WILL BE ERROR OR BUG-FREE,
# REGARDING THE SECURITY, RELIABILITY OR PERFORMANCE OF THE SOFTWARE, OR THAT ANY
# ERRORS IN THE SOFTWARE WILL BE CORRECTED.  YOUR USE OF THE SOFTWARE IS AT YOUR SOLE RISK.
# THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND WITHOUT WARRANTY OF ANY KIND.
# TO THE EXTENT PERMITTED BY LAW, LEDAS DISCLAIMS ALL WARRANTIES AND CONDITIONS OF ANY KIND,
# WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES AND CONDITIONS
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT. ANY CONTENT 
# OR MATERIAL DOWNLOADED OR OTHERWISE OBTAINED THROUGH THE USE OF THE SOTWARE IS DONE AT
# YOUR OWN DISCRETION AND RISK AND YOU WILL BE SOLELY RESPONSIBLE FOR ANY DAMAGE TO YOUR
# COMPUTER SYSTEM OR OTHER DEVICE OR LOSS OF DATA THAT RESULTS FROM THE DOWNLOAD OF ANY
# SUCH CONTENT OR MATERIAL.  YOU CAN RECOVER FROM LEDAS AND ITS SUPPLIERS ONLY DIRECT DAMAGES
# UP TO THE AMOUNT YOU PAID FOR THE SOFTWARE.  YOU CANNOT RECOVER ANY OTHER DAMAGES, INCLUDING
# CONSEQUENTIAL, LOST PROFITS, SPECIAL, INDIRECT OR INCIDENTAL DAMAGES.
 

require 'sketchup.rb'

require "ldd\\ldd_common_functions.rb"

#============================================================================================

module LDD
  
  # line-line..
  
  EQUAL_LINES = 1  
  PARALLEL_LINES = 2
  INTERSECTING_LINES = 3
  INTERSECTING_LINES_WITH_COMMON_FACE = 4
  GENERAL_CASE_LINES = 5
  
  

  class Driving_Dimension
  
    attr_reader :p1
    attr_reader :p2
    attr_accessor :vector
    attr_reader :group    
    
    attr_accessor :flags
      
    @@dr_di_counter = 1
    
    def Driving_Dimension.ldd_constraints_counter=(iNumber)
      @@dr_di_counter = iNumber
    end
      
    def initialize(p1,p2,vector,text, iOldDrivingDimensionsGroupAndTextAry = nil)
      @flags = [nil, nil, nil]
      @num_seg = 12      
      @@dr_di_counter += 1
      @dr_di_number = @@dr_di_counter
      
      @p1 = get_point(p1)
      @p2 = get_point(p2)      
      @nil_distance = (@p1 == @p2) ? true : false      
      
      ve = get_vector(vector)      
      @vector = @nil_distance ? ve : get_perpendicular_vector(@p1,@p2,ve,true)
      @nil_vector = (@vector.length == 0) ? true : false
      
      p_text = Geom.linear_combination 0.5, @p1 + @vector, 0.5, @p2 + @vector      
      entities_all = Sketchup.active_model.entities
      
      if iOldDrivingDimensionsGroupAndTextAry
        label = getLabelEntities(iOldDrivingDimensionsGroupAndTextAry) # [text, group, cPointsArray, cLinesArray]
        
        @text = label[0] ? label[0] : (entities_all.add_text text, p_text)
        @group = label[1] ? label[1] : (entities_all.add_group)
        @entities = @group.entities
        
        @cpoint1 = (label[2])[0] ? (label[2])[0] : @entities.add_cpoint(@nil_distance ? @p1 : (@p1 + @vector))
        @cpoint2 = (label[2])[1] ? (label[2])[1] : @entities.add_cpoint(@nil_distance ? @p2 : (@p2 + @vector))
        
        clines = label[3]        
        
        @cline0 = @nil_distance ? nil : ( clines.empty? ?  setCLine(nil, @p1 + @vector, @p2 + @vector) : clines.shift)        
        @cline1 = @nil_vector ? nil : (clines.empty? ? setCLine(nil, @p1, @p1 + @vector, @vector) : clines.shift)        
        @cline2 = @nil_vector ? nil : (clines.empty? ? setCLine(nil, @p2, @p2 + @vector, @vector) : clines.shift)
        
        if (not clines.empty?)
          ((@cline0 = clines.shift) if (not @cline0)) if (not clines.empty?)
          ((@cline1 = clines.shift) if (not @cline1)) if (not clines.empty?)
          ((@cline2 = clines.shift) if (not @cline2)) if (not clines.empty?)
        end
        
        @entities.eraseEntitiesLDD clines
        
        #@cline0 = @nil_distance ? nil : ( clines.empty? ?  setCLine(nil, @p1 + @vector, @p2 + @vector) : clines.shift)        
        #@cline1 = @nil_vector ? nil : (clines.empty? ? setCLine(nil, @p1, @p1 + @vector, @vector) : clines.shift)        
        #@cline2 = @nil_vector ? nil : (clines.empty? ? setCLine(nil, @p2, @p2 + @vector, @vector) : clines.shift)      
      else      
        
        @group = entities_all.add_group
        @text = entities_all.add_text text, p_text        
        @entities = @group.entities
        
        @cline0 = @nil_distance ? nil : setCLine(nil, @p1 + @vector, @p2 + @vector)
        @cline1 = @nil_vector ? nil : setCLine(nil, @p1, @p1 + @vector, @vector)
        @cline2 = @nil_vector ? nil : setCLine(nil, @p2, @p2 + @vector, @vector)
        
        @cpoint1 = @entities.add_cpoint(@nil_distance ? @p1 : (@p1 + @vector))
        @cpoint2 = @entities.add_cpoint(@nil_distance ? @p2 : (@p2 + @vector))
      end
      
      
      #@text.display_leader=false
      @text.set_attribute "LDDGUIAttributes", "Driving_Dimension", @dr_di_number      
      @group.set_attribute "LDDGUIAttributes", "Driving_Dimension", @dr_di_number      
    end     
    
    #====================================================================================
    
    def moveWithMousePosition(iConstraintType, iMousePosition, iEntity1, iEntity2 = nil)
      case iConstraintType
    
        when LDD_constraints_types::VERTEX_SOMETHING
          self.moveForVertexWithMousePosition(iMousePosition, iEntity1)
          
        when LDD_constraints_types::EDGE_LENGTH
          self.moveForEdgeWithMousePosition(iMousePosition, iEntity1)
          
        when LDD_constraints_types::ARCCURVE_RADIUS
          self.moveForArcCurveWithMousePosition(iMousePosition, iEntity1)              
          
        when LDD_constraints_types::EDGE_EDGE_DISTANCE
          self.moveForTwoParallelEdgesWithMousePosition(iMousePosition, iEntity1, iEntity2)        
          
        when LDD_constraints_types::VERTEX_EDGE_DISTANCE
          self.moveForVertexAndEdgeWithMousePosition(iMousePosition, iEntity1, iEntity2)
          
        when LDD_constraints_types::VERTEX_VERTEX_DISTANCE
          self.moveForTwoVerticesWithMousePosition(iMousePosition, iEntity1, iEntity2)    
          
        when LDD_constraints_types::EDGE_EDGE_PLANAR_ANGLE      
          self.moveForTwoGeneralCaseEdgesWithMousePosition(iMousePosition, iEntity1, iEntity2)          
          
        when LDD_constraints_types::EDGE_EDGE_ANGLE  
          self.moveForTwoGeneralCaseEdgesWithMousePosition(iMousePosition, iEntity1, iEntity2)          
          
        when LDD_constraints_types::FIXATION
          self.moveForFixationConstraintWithMousePosition(iMousePosition, iEntity1)
          
      end
    end
    
    #====================================================================================
    def createObservers
      @group.add_observer(LDDDrDiGroupObserver.new)
    
      @text.add_observer(LDDDrDiTextObserver.new)
    end
    
    #====================================================================================    
    def moveCPointToPlace(iCpoint, iEntities, iPlace)
    
      if(iCpoint && iCpoint.valid?)
        
        return if (not (iCpoint.kind_of? Sketchup::ConstructionPoint))
        
        pos = iCpoint.position
        vec = iPlace - pos
        transformation = Geom::Transformation.translation vec
        iEntities.transform_entities transformation, iCpoint
      else        
      end
    end
    
    #====================================================================================
    def setCLine(iOldCLine, iPoint1, iPoint2, iVector = nil)
      if not iVector
        iVector = iPoint1.vector_to iPoint2    
      end
      
      if iVector.length == (0.0).to_l
        return iOldCLine
      end
              
      cond = (not iOldCLine)||iOldCLine.deleted?        
      cline = ( cond ? (@entities.add_cline iPoint1, iPoint2) : iOldCLine)        
      (return nil) if (not (cline.kind_of? Sketchup::ConstructionLine))        
      (cline.stipple = "_"*4) if cond
              
      cline.position= iPoint1
      cline.direction= iVector
      cline.start= iPoint1
      cline.end= iPoint2
        
      return cline
              
    end
    #====================================================================================  
    
    def setArc(iOldArc, iCenter , xAxis, yAxis, startAngle, endAngle, iRadius)
    
      delta_angle = (endAngle - startAngle)/(@num_seg)
          
      if(not iOldArc)      
        iOldArc = []
      end
            
      help = []
        
      for counter in 0..(@num_seg-1)    
        
        angle = startAngle + delta_angle*counter  
          
        if counter == 0
          vector = (Geom.linear_combination Math.cos(angle),  xAxis, Math.sin(angle), yAxis)
          vector.length = iRadius
          startPoint = iCenter + vector
        else
          startPoint = help[0]
        end
                  
        vector = (Geom.linear_combination Math.cos(angle + delta_angle),  xAxis, Math.sin(angle + delta_angle), yAxis)    
        vector.length = iRadius
        endPoint = iCenter + vector
          
        help[0] = endPoint
        
        iOldArc[counter] = setCLine(iOldArc[counter], startPoint, endPoint)        
      end
      
      return iOldArc
    end  
    #====================================================================================  
            
    def material=(iMaterial)
      @text.material= iMaterial if(@text)     
    end
    
    def visible=(iState)	  
      state = !iState
      @cline0.hidden= state if @cline0.valid? if(@cline0)
      @cline1.hidden= state if @cline1.valid? if(@cline1)
      @cline2.hidden= state if @cline2.valid? if(@cline2)
      @text.hidden= state if @text.valid? if(@text)
      @cpoint1.hidden= state if @cpoint1.valid? if(@cpoint1)
      @cpoint2.hidden= state if @cpoint2.valid? if(@cpoint2)
      
      if(@arc)
        if(@arc[0].valid?)
          @arc.each{ |ar|
            ar.hidden= state if ar.valid?
          }
        end
      end
    end
    
    def erase!
      #(puts "OPERATION erase" )if (ENV['LDD_DEV'] == "TRUE")
          
      ent = []      
      if @group
        if @group.valid?
          ent += @group.explode
        end
      end
      
      ent.push @text      
      
      @cline0 = nil
      @cline1 = nil
      @cline2 = nil
      @cpoint1 = nil
      @cpoint2 = nil
      @arc = nil
      @group = nil
      @text = nil
      @entities = nil
      
      Sketchup.active_model.entities.eraseEntitiesLDD(ent)
      
    end
    
    #====================================================================================
            
    
    
    #====================================================================================
    
    def getID
      return @dr_di_number
    end
    

    def text=(text)
      @text.text= text.to_s
    end
    
    def text_entity
      return @text	
    end
        
    def text
      @text.text
    end
    def middle_position
      Geom.linear_combination 0.5, @p1, 0.5, @p2
    end
    
    #====================================================================================
    
    def moveForEdgeWithMousePosition(iMouse_position, iEdge)
    
      (@entities.eraseEntitiesLDD(@arc) ;  @arc = nil) if @arc
      (@entities.eraseEntitiesLDD(@arc2) ;  @arc2 = nil) if @arc2
    
      pmid = self.middle_position
      vec1 = iMouse_position - pmid
      
      p1 = iEdge.start.position
      p2 = iEdge.end.position
      
      self.setPositionOnEdge(p1,p2,vec1)
    end
    
    #====================================================================================
    
    def moveForFixationConstraintWithMousePosition(iMousePosition, iEntity)
      if iEntity.kind_of? Sketchup::Vertex
        self.moveForVertexWithMousePositionForFixation(iMousePosition, iEntity)
      end
      
      if iEntity.kind_of? Sketchup::Edge
        self.moveForEdgeWithMousePositionForFixation(iMousePosition, iEntity)
      end
      
      if iEntity.kind_of? Sketchup::ArcCurve
        self.moveForArcCurveWithMousePosition(iMousePosition, iEntity)
      end
    
    end
    
    #====================================================================================
    def moveAutoForFixationConstraint(iEntity)
      if iEntity.kind_of? Sketchup::Vertex
        self.moveAutoForVertex(iEntity)
      end
      
      if iEntity.kind_of? Sketchup::Edge
        self.moveAutoForEdgeForFixation(iEntity)
      end
      
      if iEntity.kind_of? Sketchup::ArcCurve
        self.moveAutoForArcCurve(iEntity)
      end
    end
    #====================================================================================
    
    def moveAutoForVertex(iVertex)
    
      len = @vector.length   
         
      vec = (iVertex.position.vector_to self.p1) + @vector
      
      if (len == 0)||(vec.length == 0)
        vec = get_vector([0,0,0])
      else
        vec.length = len
      end
      self.setOrdinaryPosition(iVertex.position, iVertex.position, vec)
    end    
    
    #====================================================================================
    
    def moveForEdgeWithMousePositionForFixation(iMousePosition, iEdge)
      point = iEdge.midPoint
      self.setOrdinaryPosition(point, point, (point.vector_to iMousePosition))    
    end
    
    #====================================================================================
    
    def moveAutoForEdgeForFixation(iEdge)
      point = iEdge.midPoint
      
      len = @vector.length         
      vec = (point.vector_to self.p1) + @vector
      
      if (len == 0)||(vec.length == 0)
        vec = get_vector([0,0,0])
      else
        vec.length = len
      end
      self.setOrdinaryPosition(point, point, vec)
    end
    
    #====================================================================================
    
    def moveForVertexWithMousePositionForFixation(iMouse_position, iVertex)
    
      self.setOrdinaryPosition(iVertex.position, iVertex.position, (iVertex.position.vector_to iMouse_position))
    end
    #====================================================================================
    
    def moveAutoForEdge(iEdge)
      vec = self.vector
      p1 = iEdge.start.position
      p2 = iEdge.end.position
        
      vec1 = get_perpendicular_vector(p1,p2,vec,true)
      
      self.setPositionOnEdge(p1,p2,vec1)
    end
    
    #====================================================================================
    def setPositionOnEdge(iPoint1, iPoint2, iVector )  
      
      if (iPoint1 == iPoint2)
        @nil_distance = true			
        @vector = get_vector(iVector)      
      else
        @nil_distance = false			
        @vector = get_perpendicular_vector(iPoint1,iPoint2,iVector,true)
      end	
      
      self.setOrdinaryPosition(iPoint1, iPoint2, @vector)   
    end
    #====================================================================================
    
    def setPositionOnArcCurve(iMouse_position, iCurve, iFlag_inside)
      #iFalg_inside - inside or outside acr_angle
    
      center = iCurve.center
      plane = iCurve.plane  
    
      mouse_position_on_curve_plane = iMouse_position.project_to_plane plane
    
      if(iFlag_inside)
    
        vector = mouse_position_on_curve_plane - center
    
        if(vector.length != (0.0).to_l)
          vector.length = iCurve.radius
        end
        
          self.setOrdinaryPosition(center + vector, center + vector, iMouse_position - (center + vector))
      else
    
        vector = mouse_position_on_curve_plane - center
    
        if(vector.length != (0.0).to_l)
          vector.length = iCurve.radius
        end
          self.setOrdinaryPosition(center - vector, center - vector, iMouse_position - (center - vector))
      end
    end
    
    #====================================================================================
    
    #====================================================================================
      
    def moveAutoForArcCurve(iArcCurve)

      vector = self.vector
      len = vector.length       
      
      self.moveForArcCurveWithMousePosition(self.p1 + vector, iArcCurve)
          
      if (self.vector.length != (0.0).to_l)                     
        self.vector.length= len
      end
    end

    
    #====================================================================================
    def moveForArcCurveWithMousePosition(iMouse_position, iArcCurve)
      (@entities.eraseEntitiesLDD(@arc) ;  @arc = nil) if @arc
      (@entities.eraseEntitiesLDD(@arc2) ;  @arc2 = nil) if @arc2
    
      start_v = iArcCurve.first_edge.start 
      end_v = iArcCurve.last_edge.end
    
      if(start_v.entityID == end_v.entityID)# circle                
        self.setPositionOnArcCurve(iMouse_position, iArcCurve, true)                
                        
      else#Arc                                
        #start_angle = ldd_get_angle_on_curve(curve, start_v.position)# always == 0                
        start_angle = 0.0
        end_angle = ldd_get_angle_on_curve(iArcCurve, end_v.position)
                  
        current_curve_angle = ldd_get_angle_on_curve(iArcCurve, iMouse_position)
                  
        if (start_angle <= current_curve_angle)&&(current_curve_angle <= end_angle)
          self.setPositionOnArcCurve(iMouse_position, iArcCurve, true)                                
        else
          pi = Math::PI
                    
          if(end_angle < pi)
            #curve has "blind" areas
            #they are (end_angle, pi) and (pi + end_angle, 2*pi)                   
                      
            if((end_angle < current_curve_angle)&&(current_curve_angle < pi))
              #blind
              center = iArcCurve.center
                                              
              if(current_curve_angle <= (pi + end_angle)/2)
                vec1 = end_v.position - center
                vec1.length = (iMouse_position - center).length
                self.setPositionOnArcCurve(center + vec1, iArcCurve, true)                                                        
              else
                vec1 = center - start_v.position
                vec1.length = (iMouse_position - center).length
                self.setPositionOnArcCurve(center + vec1, iArcCurve, false)
              end
                      
            elsif(( (pi + end_angle) < current_curve_angle)&&(current_curve_angle < 2*pi))                    
              #blind
              center = iArcCurve.center
                                              
              if(current_curve_angle >= (3*pi + end_angle)/2)                      
                vec1 = start_v.position - center
                vec1.length = (iMouse_position - center).length
                self.setPositionOnArcCurve(center + vec1, iArcCurve, true)
              else
                vec1 = center - end_v.position
                vec1.length = (iMouse_position - center).length
                self.setPositionOnArcCurve(center + vec1, iArcCurve, false)
              end
                        
            else  #not blind
              self.setPositionOnArcCurve(iMouse_position, iArcCurve, false)                      
            end
          else  #end_angle >= pi
            self.setPositionOnArcCurve(iMouse_position, iArcCurve, false)                                          
          end  #end_angle
        end  #start_angle......
                
      end  #Arc
    
    end  
    #====================================================================================
    def moveAutoForTwoParallelEdges(iEdge1, iEdge2)    
      self.moveForTwoParallelEdgesWithMousePosition(self.p1 + self.vector, iEdge1, iEdge2)      
    end
    #====================================================================================
    def moveForTwoParallelEdgesWithMousePosition(iMouse_position, iEdge1, iEdge2)     
    
      (@entities.eraseEntitiesLDD(@arc) ;  @arc = nil) if @arc
      (@entities.eraseEntitiesLDD(@arc2) ;  @arc2 = nil) if @arc2
    
      point1 = iMouse_position.project_to_line iEdge1.line
      point2 = iMouse_position.project_to_line iEdge2.line
      
      e1Start = iEdge1.start.position
      e1End = iEdge1.end.position
      
      e2Start = iEdge2.start.position
      e2End = iEdge2.end.position
      
      if(point1.between?(e1Start, e1End))
        @p1 = point1
        @vector = get_vector([0,0,0])
      else
        if((point1.distance e1Start)>(point1.distance e1End))
          @p1 = e1End        
          @vector = e1End.vector_to point1
        else
          @p1 = e1Start
          @vector = e1Start.vector_to point1
        end
      end
      
      if(point2.between?(e2Start, e2End))
        @p2 = point2
        #additional_vector = get_vector([0,0,0])
      else
        if((point2.distance e2Start)>(point2.distance e2End))
          @p2 = e2End         
        else
          @p2 = e2Start        
        end
                    
        #additional_vector = @vector -  (@p1.vector_to (@p2.project_to_line iEdge1.line))     
      end
      
      additional_vector = @vector -  (@p1.vector_to (@p2.project_to_line iEdge1.line))     
      normal = (@p1 + @vector).vector_to (@p2 + additional_vector)
      
      if((@p1 + @vector) == (@p2 + additional_vector))
        @nil_distance = true
      else
        @nil_distance = false
      end
      
      if (@vector.length == 0)
        @nil_vector = true
      else
        @nil_vector = false
      end
      
      if (additional_vector.length == 0)      
        nil_add_vector = true
      else
        nil_add_vector = false
      end
      
      
      
      
      
      #           * ----------------------  p2
      #           |                       .
      #           |                       . 
      #    vec    |                       .  
      # ----------* ......................* p2_project
      #p1
      

      if (not @nil_vector)
        @cline1 = setCLine(@cline1, @p1, @p1 + @vector, @vector)
      else        
        (@entities.eraseEntitiesLDD(@cline1) ;  @cline1 = nil) if @cline1
      end
      
      if (not nil_add_vector)
        @cline2 = setCLine(@cline2, @p2, @p2 + additional_vector, additional_vector)
      else
        (@entities.eraseEntitiesLDD(@cline2) ;  @cline2 = nil) if @cline2
      end
      

      if (not @nil_distance)
        moveCPointToPlace(@cpoint1, @entities, @p1 + @vector)
        moveCPointToPlace(@cpoint2, @entities, @p2 + additional_vector)
        
        @cline0 = setCLine(@cline0, @p1 + @vector, @p2 + additional_vector, normal)      
      else
        moveCPointToPlace(@cpoint1, @entities, @p1)
        moveCPointToPlace(@cpoint2, @entities, @p2)
        (@entities.eraseEntitiesLDD(@cline0) ;  @cline0 = nil) if @cline0
      end

      @text.point= Geom.linear_combination 0.5, @p1 + @vector, 0.5, @p2 + additional_vector 
    end
  #====================================================================================
  def moveForVertexWithMousePosition(iMouse_position, iVertex)
    
    self.setOrdinaryPosition(iVertex.position, iMouse_position, get_vector([0,0,0]))
  end
  #====================================================================================
  def moveForVertexAndEdgeWithMousePosition(iMouse_position, iVertexOrEdge1, iVertexOrEdge2)

    (@entities.eraseEntitiesLDD(@arc) ;  @arc = nil) if @arc
    (@entities.eraseEntitiesLDD(@arc2) ;  @arc2 = nil) if @arc2

    if(iVertexOrEdge1.kind_of? Sketchup::Vertex)
      vertex = iVertexOrEdge1
      edge = iVertexOrEdge2
    else
      vertex = iVertexOrEdge2
      edge = iVertexOrEdge1    
    end
    
    @p1 = vertex.position
    
    help_line = [vertex.position, edge.line[1]]   
    help_point = iMouse_position.project_to_line help_line  
    @vector = @p1.vector_to help_point
    
    sPoint = edge.start.position
    ePoint = edge.end.position
    
    p_proj = @p1.project_to_line edge.line
    
    if(p_proj.between?(sPoint, ePoint))
      @p2 = p_proj
    else
      if((@p2.distance sPoint)>(@p2.distance ePoint))
        @p2 = ePoint
      else
        @p2 = sPoint
      end
    end
    
    additional_vector = @p2.vector_to iMouse_position.project_to_line edge.line
    
    if((@p1 + @vector) == (@p2 + additional_vector))
      @nil_distance = true
    else
      @nil_distance = false
    end
      
    if (@vector.length == 0)
      @nil_vector = true
    else
      @nil_vector = false
    end
    
    if (additional_vector.length == 0)
      nil_add_vector = true
    else
      nil_add_vector = false
    end
    
    
    if (not @nil_vector)
      @cline1 = setCLine(@cline1, @p1, @p1 + @vector, @vector)
    else
      (@entities.eraseEntitiesLDD(@cline1) ;  @cline1 = nil) if @cline1
    end     
    
    
    if (not nil_add_vector)
      @cline2 = setCLine(@cline2, @p2, @p2 + additional_vector, additional_vector)      
    else
      (@entities.eraseEntitiesLDD(@cline2) ;  @cline2 = nil) if @cline2
    end    

    if (not @nil_distance)
      moveCPointToPlace(@cpoint1, @entities, @p1 + @vector)
      moveCPointToPlace(@cpoint2, @entities, @p2 + additional_vector)      
      @cline0 = setCLine(@cline0, @p1 + @vector, @p2 + additional_vector, (@p1.vector_to p_proj))      
    else
      moveCPointToPlace(@cpoint1, @entities, @p1)
      moveCPointToPlace(@cpoint2, @entities, @p2)
      (@entities.eraseEntitiesLDD(@cline0) ;  @cline0 = nil) if @cline0
    end
    
    @text.point= Geom.linear_combination 0.5, @p1 + @vector, 0.5, @p2 + additional_vector
  end
  #====================================================================================
  def moveAutoForVertexAndEdge(iVertexOrEdge1, iVertexOrEdge2)

    self.moveForVertexAndEdgeWithMousePosition(self.p1 + self.vector, iVertexOrEdge1, iVertexOrEdge2)
  end

    #====================================================================================

    def moveForTwoVerticesWithMousePosition(iMouse_position, iVertex1, iVertex2)
      
      (@entities.eraseEntitiesLDD(@arc) ;  @arc = nil) if @arc
      (@entities.eraseEntitiesLDD(@arc2) ;  @arc2 = nil) if @arc2
    
      point1 = iVertex1.position
      point2 = iVertex2.position
      
      midPoint = Geom.linear_combination 0.5, point1, 0.5, point2
    
      if (point1 == point2)
        
        vector = midPoint.vector_to iMouse_position      
      else      
        vector = midPoint.vector_to iMouse_position      
        vector = get_perpendicular_vector(point1,point2, vector,false)
      end	    
      
      self.setOrdinaryPosition(point1, point2, vector)
      
    end
    
    #====================================================================================

    def moveAutoForTwoVertices(iVertex1, iVertex2)
    
      point1 = iVertex1.position
      point2 = iVertex2.position
      
      midPoint = Geom.linear_combination 0.5, point1, 0.5, point2
    
      if (point1 == point2)
        
        vector = @vector
      else      
        vector = @vector
        vector = get_perpendicular_vector(point1,point2, vector,true)
      end	    
      self.setOrdinaryPosition(point1, point2, vector)
    end
    
    #============================================================================================
    def setOrdinaryPosition(iPoint1, iPoint2, iVector)
      @p1 = iPoint1
      @p2 = iPoint2
      
      if(iVector == nil)
        @vector = get_vector([0,0,0])
      else
        @vector = iVector
      end
    
      if (@p1 == @p2)
        @nil_distance = true			      
      else
        @nil_distance = false			      
      end	
      
      if (@vector.length == 0)
        @nil_vector = true
      else
        @nil_vector = false
      end
      
      if (not @nil_vector)
        @cline1 = setCLine(@cline1, @p1, @p1 + @vector, @vector)
        @cline2 = setCLine(@cline2, @p2, @p2 + @vector, @vector)      
      else
        (@entities.eraseEntitiesLDD(@cline1) ;  @cline1 = nil) if @cline1        
        (@entities.eraseEntitiesLDD(@cline2) ;  @cline2 = nil) if @cline2
      end

      if (not @nil_distance)
        moveCPointToPlace(@cpoint2, @entities, @p2 + @vector)
        moveCPointToPlace(@cpoint1, @entities, @p1 + @vector)
        
        @cline0 = setCLine(@cline0, @p1 + @vector, @p2 + @vector)
      else
        moveCPointToPlace(@cpoint1, @entities, @p1)
        moveCPointToPlace(@cpoint2, @entities, @p2)
        (@entities.eraseEntitiesLDD(@cline0) ;  @cline0 = nil) if @cline0
      end

      @text.point = Geom.linear_combination 0.5, @p1 + @vector, 0.5, @p2 + @vector
    end
   #==============================================================================================
    def moveForTwoGeneralCaseEdgesWithMousePosition(iMousePosition, iEdge1, iEdge2)
      
      @flags = iEdge1.getFlagsForTwoGeneralCaseEdgesConstraint(@flags, iEdge2, iMousePosition)
    
      case iEdge1.linesType(iEdge2)      
    
        when PARALLEL_LINES
          moveForTwoParallelEdgesWithMousePosition(iMousePosition, iEdge1, iEdge2)        
        
        when INTERSECTING_LINES        
          length =(iEdge1.getIntersectionPoint(iEdge2).vector_to iMousePosition).length          
          self.setPositionForTwoCoPlanarEdges(iEdge1, iEdge2, length, 
                                              iEdge1.normalTheirSelfTwoEdges(iEdge2, @flags, iMousePosition))
          
        when INTERSECTING_LINES_WITH_COMMON_FACE
          length =(iEdge1.getIntersectionPoint(iEdge2).vector_to iMousePosition).length          
          self.setPositionForTwoCoPlanarEdges(iEdge1, iEdge2, length)         
          
        when GENERAL_CASE_LINES
          plane = iEdge1.commonPlane(iEdge2, @flags)        
          e2StartOnPlane = iEdge2.start.position.project_to_plane plane
          e2EndOnPlane = iEdge2.end.position.project_to_plane plane      
          point0 = Geom.intersect_line_line iEdge1.line, [e2StartOnPlane, e2EndOnPlane]
        
          length =(point0.vector_to iMousePosition).length          
          self.setPositionForTwoGeneralCaseEdges(iEdge1, iEdge2, length)         
        
      end
      
      
    end
    
    #==============================================================================================        
    
    def moveAutoForTwoGeneralCaseEdges(iEdge1, iEdge2)
    
      case iEdge1.linesType(iEdge2)      
    
        when PARALLEL_LINES
          moveAutoForTwoParallelEdges(iEdge1, iEdge2)
        
        when INTERSECTING_LINES        
          length = self.vector.length          
          self.setPositionForTwoCoPlanarEdges(iEdge1, iEdge2, length, 
                                              iEdge1.normalTheirSelfTwoEdges(iEdge2, @flags))
          
        when INTERSECTING_LINES_WITH_COMMON_FACE
          length = self.vector.length          
          setPositionForTwoCoPlanarEdges(iEdge1, iEdge2, length)     
          
        when GENERAL_CASE_LINES
          length = self.vector.length          
          self.setPositionForTwoGeneralCaseEdges(iEdge1, iEdge2, length)
        
      end
      
      
    end
    
    #============================================================================================
    def setPositionForTwoGeneralCaseEdges(iEdge1, iEdge2, iLength)
      (iLength = (1.0).to_l) if (iLength < (1.0).to_l)
                
      plane = iEdge1.commonPlane(iEdge2, @flags)
            
      e2Start = iEdge2.start.position
      e2End = iEdge2.end.position      
            
      e2StartOnPlane = e2Start.project_to_plane plane
      e2EndOnPlane = e2End.project_to_plane plane
      
      point01 = Geom.intersect_line_line iEdge1.line, [e2StartOnPlane, e2EndOnPlane]
      
      normal = e2StartOnPlane.vector_to e2Start      
      
      point02 = point01 + normal                 
      normal = plane[1]

      edge1Vector = iEdge1.vectorLDD(@flags[1])
      edge1Vector.length= iLength
      
      edge2Vector = iEdge2.vectorLDD(@flags[2])
      edge2Vector.length= iLength
      
      point1 = (@flags[1] ? iEdge1.start.position : iEdge1.end.position)
      point2 = (@flags[1] ? iEdge1.end.position : iEdge1.start.position)
            
      point3 = (@flags[2] ? iEdge2.start.position : iEdge2.end.position)
      point4 = (@flags[2] ? iEdge2.end.position : iEdge2.start.position)
                                     
      @p1 = point01
      @p2 = point2
      @vector = Geom::Vector3d.new edge1Vector              
      
      curveEndOnEdge1 = point01 + edge1Vector      
      curveEndOnEdge2 = point02 + edge2Vector
        
      cLine1StartPosition = (curveEndOnEdge1.between?(point1, point2)) ? curveEndOnEdge1 :
                                       ((curveEndOnEdge1.between?(point01, point1)) ? point1 : point2)
        
      vector1 = cLine1StartPosition.vector_to curveEndOnEdge1
      
      cLine2StartPosition = (curveEndOnEdge2.between?(point3, point4)) ? curveEndOnEdge2 :
                                       ((curveEndOnEdge2.between?(point02, point3)) ? point3 : point4)
      
      vector2 = cLine2StartPosition.vector_to curveEndOnEdge2
      
      nil_vector1 = (vector1.length == (0.0).to_l) ? true : false              
      nil_vector2 = (vector2.length == (0.0).to_l) ? true : false
                  
      if (not nil_vector1)
        @cline1 = setCLine(@cline1, cLine1StartPosition, curveEndOnEdge1, vector1)      
      else
        (@entities.eraseEntitiesLDD(@cline1) ;  @cline1 = nil) if @cline1
      end      
      
      if (not nil_vector2)
        @cline2 = setCLine(@cline2, cLine2StartPosition, curveEndOnEdge2, vector2)            
      else
        (@entities.eraseEntitiesLDD(@cline2) ;  @cline2 = nil) if @cline2
      end      
                  
      moveCPointToPlace(@cpoint1, @entities, curveEndOnEdge1)
      moveCPointToPlace(@cpoint2, @entities, curveEndOnEdge2)
                  
      edge2Angle = edge1Vector.angle_between edge2Vector
      
      yAxis = normal * edge1Vector
      
      edge1Vector.normalize!
      yAxis.normalize!          
      
      vector = (Geom.linear_combination Math.cos(edge2Angle/2),  edge1Vector, Math.sin(edge2Angle/2), yAxis)
      vector.length = iLength
      @text.point = (Geom.linear_combination 0.5, point01, 0.5, point02) + vector
      
      @cline0 = setCLine(@cline0, point01 + vector, point02 + vector)      
      #(@entities.eraseEntitiesLDD(@cline0) ;  @cline0 = nil) if @cline0          
      
      start_angle = 0.0
      end_angle = edge2Angle      
      
      #length - radius
      #second_edge_angle - angle
      #@p1 + edge1Vector - Xaxis      
          
      @arc = setArc(@arc, point01, edge1Vector, yAxis, start_angle, start_angle + (end_angle - start_angle)/2, iLength)      
      @arc2 = setArc(@arc2, point02, edge1Vector, yAxis, start_angle + (end_angle - start_angle)/2, end_angle, iLength)      
    end
    
    #============================================================================================
    def setPositionForTwoCoPlanarEdges(iEdge1, iEdge2, iLength, iNormal = nil)
      (iLength = (1.0).to_l) if (iLength < (1.0).to_l)
    
      (@entities.eraseEntitiesLDD(@arc2) ;  @arc2 = nil) if @arc2
    
      point0 = iEdge1.getIntersectionPoint(iEdge2)      
      
      edge1Vector = iEdge1.vectorLDD(@flags[1])
      edge1Vector.length= iLength
      
      edge2Vector = iEdge2.vectorLDD(@flags[2])
      edge2Vector.length= iLength
      
      point1 = (@flags[1] ? iEdge1.start.position : iEdge1.end.position)
      point2 = (@flags[1] ? iEdge1.end.position : iEdge1.start.position)
      
      point3 = (@flags[2] ? iEdge2.start.position : iEdge2.end.position)
      point4 = (@flags[2] ? iEdge2.end.position : iEdge2.start.position)
      
      (@entities.eraseEntitiesLDD(@cline0) ;  @cline0 = nil) if @cline0
          
      @p1 = point0      
      @p2 = point2
      @vector = Geom::Vector3d.new edge1Vector
      
      
      
      
      #                 /
      #                * points[2]                  * mouseOnPlane
      #               /A                            | 
      #      edge2   / |                            |
      #             /  | vectorToPoint2(yaxis)      |
      #            /   |                            | 
      #           /    |                            | 
      #          /     |                            |
      #         /      |                            |(mouseOnEdge1)
      #        /       |     edge1(xaxis)           |
      #       *------------------------------*------*---------- (line = [@p1, edge1Vector])
      #   @p1=points[0]                 @p2=points[1]
      #
      
              
      
      curveEndOnEdge1 = point0 + edge1Vector 
      curveEndOnEdge2 = point0 + edge2Vector 
      
      
      cLine1StartPosition = (curveEndOnEdge1.between?(point1, point2)) ? curveEndOnEdge1 :
                                       ((curveEndOnEdge1.between?(point0, point1)) ? point1 : point2)
        
      vector1 = cLine1StartPosition.vector_to curveEndOnEdge1
      
      cLine2StartPosition = (curveEndOnEdge2.between?(point3, point4)) ? curveEndOnEdge2 :
                                       ((curveEndOnEdge2.between?(point0, point3)) ? point3 : point4)
      
      vector2 = cLine2StartPosition.vector_to curveEndOnEdge2
      
      nil_vector1 = (vector1.length == (0.0).to_l) ? true : false              
      nil_vector2 = (vector2.length == (0.0).to_l) ? true : false
            
      if (not nil_vector1)
        @cline1 = setCLine(@cline1, cLine1StartPosition, curveEndOnEdge1, vector1)      
      else
        (@entities.eraseEntitiesLDD(@cline1) ;  @cline1 = nil) if @cline1
      end      
      
      if (not nil_vector2)
        @cline2 = setCLine(@cline2, cLine2StartPosition, curveEndOnEdge2, vector2)            
      else
        (@entities.eraseEntitiesLDD(@cline2) ;  @cline2 = nil) if @cline2
      end
      
      moveCPointToPlace(@cpoint1, @entities, curveEndOnEdge1)
      moveCPointToPlace(@cpoint2, @entities, curveEndOnEdge2)
      
      if not iNormal      
        face = iEdge1.common_face iEdge2      
        return if not face      
        iNormal = face.normal
      end
      
      edge2Angle = edge1Vector.getPlanarAngleForVector(edge2Vector, iNormal)
      
      yAxis = iNormal * edge1Vector
      
      edge1Vector.normalize!
      yAxis.normalize!
      
      vector = @flags[0] ?  (Geom.linear_combination Math.cos(edge2Angle/2),  edge1Vector, Math.sin(edge2Angle/2), yAxis) :
                            (Geom.linear_combination Math.cos(edge2Angle/2),  edge1Vector, Math.sin(edge2Angle/2), yAxis)
                            
      vector.length = iLength
      
      @text.point = @flags[0] ? (point0 + vector) : (point0 - vector)
      
      start_angle = @flags[0] ? 0.0 : edge2Angle
      end_angle = @flags[0] ? edge2Angle : 2*Math::PI
                  
      #length - radius
      #second_edge_angle - angle
      #@p1 + edge1Vector - Xaxis      
          
      @arc = setArc(@arc, @p1, edge1Vector, yAxis, start_angle, end_angle, iLength)   
    end
    #============================================================================================  
  end
end

file_loaded("ldd_class.rb")
