# Copyright 2008, LEDAS

# NO PART OF THIS SOFTWARE MAY BE RE-USED FOR ANY OTHER PURPOSE THAN TO RUN
# DRIVING DIMENSIONS WITHOUT THE EXPRESS AND WRITTEN APPROVAL OF LEDAS.


# YOU UNDERSTAND AND AGREE THAT LEDAS IS MAKING NO WARRANTY THAT THE SOFTWARE WILL
# MEET YOUR REQUIREMENTS, THAT THE SOFTWARE WILL BE ERROR OR BUG-FREE,
# REGARDING THE SECURITY, RELIABILITY OR PERFORMANCE OF THE SOFTWARE, OR THAT ANY
# ERRORS IN THE SOFTWARE WILL BE CORRECTED.  YOUR USE OF THE SOFTWARE IS AT YOUR SOLE RISK.
# THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND WITHOUT WARRANTY OF ANY KIND.
# TO THE EXTENT PERMITTED BY LAW, LEDAS DISCLAIMS ALL WARRANTIES AND CONDITIONS OF ANY KIND,
# WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES AND CONDITIONS
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT. ANY CONTENT 
# OR MATERIAL DOWNLOADED OR OTHERWISE OBTAINED THROUGH THE USE OF THE SOTWARE IS DONE AT
# YOUR OWN DISCRETION AND RISK AND YOU WILL BE SOLELY RESPONSIBLE FOR ANY DAMAGE TO YOUR
# COMPUTER SYSTEM OR OTHER DEVICE OR LOSS OF DATA THAT RESULTS FROM THE DOWNLOAD OF ANY
# SUCH CONTENT OR MATERIAL.  YOU CAN RECOVER FROM LEDAS AND ITS SUPPLIERS ONLY DIRECT DAMAGES
# UP TO THE AMOUNT YOU PAID FOR THE SOFTWARE.  YOU CANNOT RECOVER ANY OTHER DAMAGES, INCLUDING
# CONSEQUENTIAL, LOST PROFITS, SPECIAL, INDIRECT OR INCIDENTAL DAMAGES.


require 'sketchup.rb'

require "ldd\\ldd_constraints.rb"
require "ldd\\ldd_sketchup_classes.rb"

#====================================================================================

module LDD

  def getLabelEntities(iOldDrivingDimensionsGroupAndTextAry)
    group = iOldDrivingDimensionsGroupAndTextAry[0]
    text = iOldDrivingDimensionsGroupAndTextAry[1]
    
    to_return = [] # [text, group, cline0, cline1, cline2, cpoint1, cpoint2, arc1, arc2]
                   # [text, group, cPointsArray, cLinesArray]
    
    to_return[0] = text
    to_return[1] = group
    to_return[2] = []
    to_return[3] = []
    
    (return to_return) if (not group)
    
    group.entities.each{ |entity|
      if entity.kind_of? Sketchup::ConstructionLine
        to_return[3].push entity
      end
      
      if entity.kind_of? Sketchup::ConstructionPoint
        to_return[2].push entity
      end
    }
    
    return to_return    
  end

  def DEG2RAD(iDegrees) 
    return (iDegrees/180)*(Math::PI)
  end

  def RAD2DEG(iRadians) 
    return (iRadians/(Math::PI))*180
  end
  #====================================================================================
  class LDD_return_values
    
    LGS3D_ERROR = 1
    LGS3D_INTERRUPTED_CALCULATION = 2
    LGS3D_INVALID_OBJECT = 3
    LGS3D_OUT_OF_RANGE = 4
    LGS3D_NOT_SOLVED = 5
    LGS3D_INCONSISTENT = 6

    LGS3D_EVALUATION_PERIOD_EXPIRED = 7
    
    LDD_OK = 0
    LDD_NO_CONSTRAINTS = 14 #empty model
    LDD_INTERNAL_ERROR = 15  
    LDD_ADD_FACE_ERROR = 16 #crash face
    LDD_UPDATE_ERROR = 17 #LGS OK , but LDD KO
    LDD_TYPED_INCORRECT_VALUE = 18
    
  end
  

  #====================================================================================
  def ldd_message_Nothing_to_Solve
    UI.messagebox  "DRIVING DIMENSIONS \n \n"+
                   "Nothing to solve. \n" +
                   "Please, create a driving dimension first."
  end
  #====================================================================================
  def ldd_message_LGS_KO
    UI.messagebox  "Your model has no solutions. \n" +
       "Please change the dimension values. \n" + 
       "If you think it is a bug, please submit it to \n" +
       "http://drivingdimensions.com/forum" 
  end
  #====================================================================================
  def ldd_message_Cannot_Apply
    UI.messagebox "DRIVING DIMENSIONS \n \n"+
                  "Your driving dimension values cannot be applied to the model. \n"+ 
                  "Please change the dimension values.\n"+
                  "If you think it is a bug, please submit it to \n"+
                  "http://drivingdimensions.com/forum"
    
  end
  #====================================================================================
  def ldd_message_Topology_Crash
    UI.messagebox "DRIVING DIMENSIONS \n \n"+
                  "Your model has been updated to satisfy the given driving dimensions, \n"+
                  "but some faces have been missed. \n"+
                  "Please change the dimension values to have a correct solution. \n"+
                  "If you think it is a bug, please submit it to \n"+
                  "http://drivingdimensions.com/forum"
  end
  #====================================================================================
  def ldd_message_Internal_Error
    UI.messagebox "DRIVING DIMENSIONS \n \n"+
                  "Internal error. \n"+
                  "Please check that you have correctly installed \n"+
                  "the Driving Dimensions plug-in. \n"+
                  "If you think it is a bug, please submit it to \n"+
                  "http://drivingdimensions.com/forum"
  end
  #====================================================================================
  def ldd_message_Input_Error
    UI.messagebox "DRIVING DIMENSIONS \n \n"+
                  "Incorrect value given. \n"+                
                  "If you think it is a bug, please submit it to \n"+
                  "http://drivingdimensions.com/forum"
  end
  #====================================================================================
  def ldd_error_messagebox(iErrorNumber)
    if (not ENV['LDD_BATCH_TESTING'] == "TRUE")
    
      case iErrorNumber  
      
        when LDD_return_values::LGS3D_ERROR
            ldd_message_LGS_KO
        
        when LDD_return_values::LGS3D_INTERRUPTED_CALCULATION 
            ldd_message_LGS_KO
        
        when LDD_return_values::LGS3D_INVALID_OBJECT
            ldd_message_LGS_KO
        
        when LDD_return_values::LGS3D_OUT_OF_RANGE
            ldd_message_LGS_KO
        
        when LDD_return_values::LGS3D_NOT_SOLVED
            ldd_message_LGS_KO
        
        when LDD_return_values::LGS3D_INCONSISTENT
            ldd_message_LGS_KO

        when LDD_return_values::LGS3D_EVALUATION_PERIOD_EXPIRED
            ldd_message_LGS_KO
    
        when LDD_return_values::LDD_OK          
        
        when LDD_return_values::LDD_NO_CONSTRAINTS
            ldd_message_Nothing_to_Solve
        
        when LDD_return_values::LDD_INTERNAL_ERROR
            ldd_message_Internal_Error
            
        when LDD_return_values::LDD_ADD_FACE_ERROR
            ldd_message_Topology_Crash
        
        when LDD_return_values::LDD_UPDATE_ERROR
            ldd_message_Cannot_Apply
        
        when LDD_return_values::LDD_TYPED_INCORRECT_VALUE    
            ldd_message_Input_Error
        else  
     
      end  
    end
  end
  #====================================================================================
  def ldd_tool_validation_proc()
    case Driving_Dimensions_Tool.active?
      when 1
        return MF_CHECKED
      when -1
        return MF_UNCHECKED
      when 0
        return MF_UNCHECKED #MF_GRAYED
    end
  end
  
  #====================================================================================
  
  def ldd_fixation_tool_validation_proc()
    case Driving_Dimensions_Fixation_Tool.active?
      when 1
        return MF_CHECKED
      when -1
        return MF_UNCHECKED
      when 0
        return MF_UNCHECKED #MF_GRAYED
    end
  end

  #====================================================================================
  def ldd_hide_show_validation_proc()
    if (not LDDConstraints.visible?)
      return MF_CHECKED
    else
      return MF_UNCHECKED
    end
  end
  #====================================================================================
   
  def ldd_mode_name(mode)
	  case mode
		  when 51 then return "VxE_ExF"
		  when 52 then return "VxE_VxF"
		  when 53 then return "VxF_ExF"
		  when 54 then return "VxE_VxF_ExF"
	  end
  end
  #====================================================================================
  def ldd_hide_show()
    Sketchup.active_model.start_operation_LDD "Hide/Show Driving Dimensions"
    
    if(LDDConstraints.visible?)
      LDDConstraints.visible= false
      
      if (ENV['LDD_DEV'] == "TRUE")
        puts "set visible = false"    
        puts LDDConstraints.visible?.to_s
      end
      
    else
      LDDConstraints.visible= true
      
      if (ENV['LDD_DEV'] == "TRUE")
        puts "set visible = true"    
        puts LDDConstraints.visible?.to_s
      end
      
    end
    
    state = LDDConstraints.visible?
    
    LDDConstraint.constraints.each { |constraint|
      constraint.drivingDimensionEntity.visible= state
    }    
    
    LDDConstraint.constraints.recalculateLabelsPositions(false) if state    
    Sketchup.active_model.commit_operation_LDD	#hide/show
  end

  #====================================================================================
  def recalculateLabelsPositionsTwoTimes
    Sketchup.active_model.start_operation_LDD	"Recalculate labels positions"                     
    LDDConstraint.constraints.recalculateLabelsPositions(false)
    LDDConstraint.constraints.recalculateLabelsPositions(false)
    Sketchup.active_model.commit_operation_LDD
  end
  #====================================================================================
  def ldd_recalculate_model

      Sketchup.active_model.start_operation_LDD "Solve model"
      result = ldd_solve("imm_solve")			
      if (result == 0)
          Sketchup.active_model.commit_operation_LDD
          #puts LDDLabelsObserverForVertex.changed?.to_s
          LDDConstraint.constraints.recalculateLabelsPositions(true) if LDDLabelsObserverForVertex.changed?
      else
           Sketchup.active_model.abort_operation_LDD	    
      end  

  end

  #====================================================================================
  def ldd_solve(iMessage)
    
    if (not iMessage)
      imm = LDDConstraint.immediatelySolve
    else
      imm = true    
    end

    if (imm)
    
      mod = Sketchup.active_model    

      mydll = Sketchup.find_support_file("\Plugins") + "/ldd/ldd.so"
      if (ENV['LDD_DEV'] == "TRUE")
        puts "+++++Path to loaded dll: \n" + mydll.to_s
      end

      if (!FileTest.exist?( mydll ) )
        ldd_error_messagebox(LDD_return_values::LDD_INTERNAL_ERROR)      
        return LDD_return_values::LDD_INTERNAL_ERROR
      end

      require mydll
      ldd_solver = LDD_SOLVER.new

      mode = mod.get_attribute "LDDAttributes", "mode", 52	
      ary = LDDConstraint.constraints
      iArray = [mode,ary]   
      
      returned = ldd_solver.LDDStart(mod, iArray)    
      # returned - array of returned information
      
      solved = returned[0]
      # equals 0 if sucsessful
      
      if (solved != 0)
        ldd_error_messagebox(solved)      
      else                    
      end
      
      if (ENV['LDD_DEV'] == "TRUE")
        returned.each { |ret|
          puts ret.to_s
        }
      end    
      return solved
    else    
      return 0
    end
    #return solved
  end

  #====================================================================================

  def get_vector(vector)

    if (vector == nil)
      return Geom::Vector3d.new(0,0,0)
    end

    if (vector.kind_of? Array)
      return Geom::Vector3d.new(vector)
    end
    
    if (vector.kind_of? Geom::Vector3d)
      return vector
    end
    
    return Geom::Vector3d.new(0,0,0)	
  end

  def get_point(point)

    if (point.kind_of? Array)
      return Geom::Point3d.new(point)
    end

    if (point.kind_of? Geom::Point3d)
      return point
    end
    
    return point
  end


  def get_perpendicular_vector(p1,p2,vector,flag = false)
  # flag = true if you want to save length
    if (vector.length == 0)
      return vector
    end
    line = [p1,p2]
    p3 = p1 + vector
    p4 = p3.project_to_line line
    v2 = p4 - p1
    v0 = vector - v2
    if(not flag)
      return v0
    else
      if(v0.length != 0)	
        v0.length= vector.length
      end
      return v0
    end
    
  end
  #====================================================================================
    
  def findEntitiOnIdAttributeOnArray(iID, iArrayOfEntities)
    iArrayOfEntities.each{ |entity|    
      return entity if ((entity.get_attribute "LDDGUIAttributes", "Driving_Dimension", 0) == iID)      
    }
    return nil  
  end
  
  
  def ldd_remake_labels_FindSecondEntityOnIdInArray(iID, iArrayOfRestr) #[type, value, id, flag, entity]            
    iArrayOfRestr.each{ |restr|    
      if(restr[3] == 2)
        return restr[4] if( restr[2] == iID)
      end      
    }
    return nil  
  end
  
  def getDefaultPoint(iConstraintType, iEntity, iEntity2 = nil)
    case iConstraintType    
        when LDD_constraints_types::VERTEX_SOMETHING
          return iEntity.position
          
        when LDD_constraints_types::EDGE_LENGTH
          faces = iEntity.faces
          
          p1 = iEntity.start.position
          p2 = iEntity.end.position
          
          if (faces[0] !=nil)
            normal = faces[0].normal
            vec = (p2 - p1)*normal
            vec.length = ((p1 - p2).length)/2
            
            return (iEntity.midPoint + vec)
          else
            vec = Geom::Vector3d.new([1,1,1])
            vec.length = ((p1 - p2).length)/2
            
            return (iEntity.midPoint + vec)
          end
          
        when LDD_constraints_types::ARCCURVE_RADIUS
          p = iEntity.vertices[((iEntity.count_edges)/2).to_i].position		      
          vec = p - iEntity.center	
          
          return p + vec
          
        when LDD_constraints_types::EDGE_EDGE_DISTANCE
          return iEntity.midPoint
          
        when LDD_constraints_types::VERTEX_EDGE_DISTANCE
          return iEntity.position
          
        when LDD_constraints_types::VERTEX_VERTEX_DISTANCE
          return iEntity.position
          
        when LDD_constraints_types::EDGE_EDGE_PLANAR_ANGLE      
          points = iEntity.getPointsTwoEdgesForAngularConstraint(iEntity2)
          
          if points[0]          
            return Geom.linear_combination 0.5, points[1], 0.5, points[2]
          else
            return iEntity.midPoint
          end
        when LDD_constraints_types::FIXATION
          (return iEntity1.position) if (iEntity1.kind_of? Sketchup::Vertex)          
          (return getDefaultPoint(LDD_constraints_types::EDGE_LENGTH, iEntity1)) if (iEntity1.kind_of? Sketchup::Edge)
          (return getDefaultPoint(LDD_constraints_types::ARCCURVE_RADIUS, iEntity1)) if (iEntity1.kind_of? Sketchup::ArcCurve)          
      end
  
  end

  #====================================================================================

  def ldd_remake_labels(model)        
    version = model.get_attribute "LDD_VERSION", "LDD_VERSION", nil
    
    Driving_Dimension.ldd_constraints_counter=(0)      
    LDDConstraint.constraints=(LDDConstraints.new)
    
    to_erase_texts = []
    to_erase_groups = []
    to_create = []
    to_constraint = []
    entities = model.entities
    
    
    if(version == nil)  #0.1
      caseState = 1
      
      constClass = LDD_constraints_typesVersion_0_2
    
    elsif((version - 0.2).abs < 0.001)  #0.2
      caseState = 2
      
      attrName = "Restrictions"
      constClass = LDD_constraints_typesVersion_0_2
    
    elsif(version >= 0.29) #>= 0.3
      caseState = 3
      constClass = LDD_constraints_types
      
      attrName = "Constraints"
    
    end
    
    puts "state " + caseState.to_s
    
    
    case caseState
    
      when 1
      
        entities.each {|entity| 
          
          if(entity.kind_of? Sketchup::Edge) 
            curve = entity.curve
            
            if(curve == nil)	    
              length = entity.get_attribute "LDDRestrictions", "length", -1.0		    
              
              if(length > -0.5)                  
                edge_id = entity.get_attribute "LDDGUIAttributes", "Driving_Dimension", 0
                
                to_create.push [constClass::EDGE_LENGTH, length, edge_id, 1, entity] #[type, length, edge_id, flag, entity]            
              end
            else #curve		  
            
              if((curve.kind_of? Sketchup::ArcCurve)&&(not (curve.get_attribute "LDDAttributes", "added", false)))		    
                curve.set_attribute "LDDAttributes", "added", true
                radius = curve.get_attribute "LDDRestrictions", "radius", -1.0
              
                  if(radius > -0.5)
                    curve_id = curve.get_attribute "LDDGUIAttributes", "Driving_Dimension", 0
                      
                    to_create.push [constClass::ARCCURVE_RADIUS, radius.to_l, curve_id, 1, curve] #[type, length, edge_id, flag, entity]            
                  end #radius
              end	#added		  
            end #curve 		    
          end #edge
           
          if(entity.kind_of? Sketchup::Text)
            if((entity.get_attribute "LDDGUIAttributes", "Driving_Dimension", 0) > 0)
              to_erase_texts.push entity
            end      
          end  
          
          if(entity.kind_of? Sketchup::Group)
            if((entity.get_attribute "LDDGUIAttributes", "Driving_Dimension", 0) > 0)
              to_erase_groups.push entity
            end      
          end  
        }      
        puts "to create end" if (ENV['LDD_DEV'] == "TRUE")           
         
        ldd_clean_attributes(entities, "LDDRestrictions")
        
      when 2..3
        entities.each {|entity| 
        
          entityArray = []
      
          if(entity.kind_of? Sketchup::Edge) 
            curve = entity.curve
          
            if(curve == nil)
              if not ((entity.smooth?)&&(entity.soft?)) #not on cylinder edge
                [entity, entity.start, entity.end].each{ |edgeOrVertex|
              
                  if(not edgeOrVertex.get_attribute "LDDGUIAttributes", "added", false)          
                    edgeOrVertex.set_attribute "LDDGUIAttributes", "added", true                     
                    
                    entityArray.push edgeOrVertex                
                  end
                }
              end            
            else #curve
              if((curve.kind_of? Sketchup::ArcCurve)&&(not (curve.get_attribute "LDDGUIAttributes", "added", false)))		    
                curve.set_attribute "LDDGUIAttributes", "added", true            
                
                entityArray.push curve              
              end
            end
          end     
          
          entityArray.each{ |entity|
            constraints = entity.get_attribute "LDDGUIAttributes", attrName, nil    
            
            if constraints
                
              constraints.each{ |restr|
                to_create.push (restr + [entity]) #[type, value, id, flag, entity]            
              }  #each constraint  
            end #has restrictons
          }              
         
          if(entity.kind_of? Sketchup::Text)
            if((entity.get_attribute "LDDGUIAttributes", "Driving_Dimension", 0) > 0)
              to_erase_texts.push entity
            end      
          end
          
          if(entity.kind_of? Sketchup::Group)
            if((entity.get_attribute "LDDGUIAttributes", "Driving_Dimension", 0) > 0)
              to_erase_groups.push entity
            end      
          end
        }
        
    end    
    
    
     to_create.each { |constraint|     #[type, value, id, flag, entity]            
    
      if(constraint[3] == 1)
        
        ary = [nil, constraint[1], nil, constraint[4], nil] #[type,value,point,entity1,entity2]
        
        if constraint[0] >= constClass::BINARY_CONSTRAINTS_THERESHOLD
          ary[4] = ldd_remake_labels_FindSecondEntityOnIdInArray(constraint[2], to_create)
          
          if (not ary[4])
            (puts "can not find second for id= " + constraint[2].to_s) if (ENV['LDD_DEV'] == "TRUE")         
            next 
          end
        end
        
        ary[0] = (caseState < 3) ? getConstraintType(ary[3], ary[4]) : constraint[0]
        
        #search for text
        textEntity = findEntitiOnIdAttributeOnArray(constraint[2], to_erase_texts)
        (to_erase_texts.delete textEntity) if textEntity
        
        groupEntity = findEntitiOnIdAttributeOnArray(constraint[2], to_erase_groups)
        (to_erase_groups.delete groupEntity) if groupEntity
        
        if textEntity
          ary[2] = [textEntity.point, groupEntity, textEntity]
        else
          ary[2] = [getDefaultPoint(ary[0], ary[3], ary[4]) , groupEntity]
        end
        
        to_constraint.push ary          
      end    
    }
    
    ldd_clean_attributes(entities, "LDDGUIAttributes")  
    
    if (ENV['LDD_DEV'] == "TRUE")        
      to_create.each{ |constraint|      #[type, value, id, flag, entity]            
        puts constraint[0].to_s + " " + constraint[1].to_s + " " + constraint[2].to_s + " " + constraint[3].to_s + " " + constraint[4].to_s
      }
    end  
    
    if (ENV['LDD_DEV'] == "TRUE")
      puts "to erase start" 
             
      to_erase_texts.each { |entity|      
        puts entity.to_s    
      }    
      
      to_erase_groups.each { |entity|      
        puts entity.to_s    
      }    
      puts "to erase end"
    end    
    
    entities.erase_entities to_erase_texts
    entities.erase_entities to_erase_groups
    
    (puts "Constraints to create= " + to_constraint.length.to_s) if (ENV['LDD_DEV'] == "TRUE")    
    
    if (ENV['LDD_DEV'] == "TRUE")    
    
      to_constraint.each{ |constraint|      #[type,value,point,entity1,entity2]
        puts constraint[0].to_s + " " + constraint[1].to_s + " " + constraint[2].to_s + " " + constraint[3].to_s + " " + constraint[4].to_s
      }
    end
    
    to_constraint.each{ |constraint|      #[type,value,point,entity1,entity2]
      preConstraint = LDDPreConstraint.new(constraint[3], nil, (constraint[2])[1..2])
            
      preConstraint.entity2 = constraint[4]      
      preConstraint.type = constraint[0]
      preConstraint.setCurrentPosition((constraint[2])[0], nil)
      
      preConstraint.solveThenConstraintDone = false
      preConstraint.nextStep((constraint[2])[0], nil, constraint[1])
    }            
    LDDConstraint.constraints.recalculateLabelsPositions
    
    LDDConstraint.constraints.each { |constraint|
      constraint.drivingDimensionEntity.visible= true
    }                  
  end

  #====================================================================================
  def ldd_clean_attributes(iEntities, iDictName, iAttrName = nil)
    iEntities.each{ |entity|
    
      enttitiesAry = []    
      if(entity.kind_of? Sketchup::Edge)         
        curve = entity.curve                
        
        if(curve == nil)	               
          
            enttitiesAry[0] = entity
            enttitiesAry[1] = entity.start
            enttitiesAry[2] = entity.end
          
        else #curve
            if(curve.kind_of? Sketchup::ArcCurve)
              enttitiesAry[0] = curve
            end        
        end
        
        enttitiesAry.each{ |operatedEntity|
          
          if iAttrName
            operatedEntity.delete_attribute iDictName, iAttrName
          else
            operatedEntity.delete_attribute iDictName
          end
        }        
      end #edge  
    }#entities
  end
  
  #===============================================================================================
  
  
  def ldd_remake_attributesInSpecialEntitiesArray(iArray)

    iArray.uniq!
    
    constraintsFull = LDDConstraint.constraints
    
    iArray.each{ |operatedEntity|
      if(not operatedEntity.get_attribute "LDDGUIAttributes", "seen", false)  
        operatedEntity.set_attribute "LDDGUIAttributes", "seen", true
        
        constraints = operatedEntity.get_attribute "LDDGUIAttributes", "Constraints", nil
        constraintsActual = []
        
        if constraints
          
          (puts (" \n" + operatedEntity.typename + "  entID " +  operatedEntity.entityID.to_s + "has Constraints") )if (ENV['LDD_DEV'] == "TRUE")
                  
          constraints.each{ |constraint| 
            
            fullConstraint = constraintsFull.findConstraintOnID(constraint[2])
            
            puts "search" if (ENV['LDD_DEV'] == "TRUE")
            
            if fullConstraint
              puts "found full" if (ENV['LDD_DEV'] == "TRUE")
              
              (puts " ent " + fullConstraint.entities[constraint[3] - 1].to_s )if (ENV['LDD_DEV'] == "TRUE")
                         
              if(fullConstraint.entities[constraint[3] - 1] == operatedEntity) #correct attribute
                puts " +++++++ correct" if (ENV['LDD_DEV'] == "TRUE")
                constraintsActual.push constraint
              else
                puts " ------ INcorrct" if (ENV['LDD_DEV'] == "TRUE")
              end
            end          
          }
          
          operatedEntity.set_attribute "LDDGUIAttributes", "Constraints", constraintsActual          
          
          puts (operatedEntity.get_attribute "LDDGUIAttributes", "Constraints", nil).to_s if (ENV['LDD_DEV'] == "TRUE")
          
        end #restructions
      end #seen
    } #entitiesAry
    
    ldd_clean_attributes(iArray, "LDDGUIAttributes", "seen")
  end

  #===============================================================================================
  def ldd_get_angle_on_curve(iCurve,iMouse_position)  #return angle in [0, 2pi]

    center = iCurve.center
    plane = iCurve.plane
    normal = iCurve.normal
    
    mouse_position_on_curve_plane = iMouse_position.project_to_plane plane
    
    current_vector_on_plane = mouse_position_on_curve_plane - center
                  
    xaxis = iCurve.first_edge.start.position - iCurve.center 
    
    return xaxis.getPlanarAngleForVector(current_vector_on_plane, normal)
  end

  #====================================================================================
  def ldd_create_observers_on_dr_di(iDr_di)

    iDr_di.group_entity.add_observer(LDDDrDiGroupObserver.new)
    
    iDr_di.text_entity.add_observer(LDDDrDiTextObserver.new)

  end
  
  #====================================================================================
  
  #====================================================================================

  class LDDDrDiEntityObserver < Sketchup::EntityObserver

    def onEraseEntity(entity)
    
      if (ENV['LDD_DEV'] == "TRUE")
        puts "--Entity err"
      end
    
      LDDConstraint.constraints.removeCrashedConstraints
      
      if (ENV['LDD_DEV'] == "TRUE")
        puts "refresh end entity"
      end
    end  
    
  end


  #====================================================================================

  class LDDDrDiTextObserver < Sketchup::EntityObserver

    def onEraseEntity(text)
    
      if (ENV['LDD_DEV'] == "TRUE")
        puts "--Text err"
      end
    
      #(UI.messagebox  "text Observer" )if (ENV['LDD_DEV'] == "TRUE")    
      LDDConstraint.constraints.removeCrashedConstraints
      
      if (ENV['LDD_DEV'] == "TRUE")
        puts "refresh end text"
      end
    end  
    
  end

  #====================================================================================

  class LDDDrDiGroupObserver < Sketchup::EntityObserver

    def onEraseEntity(group)
      if (ENV['LDD_DEV'] == "TRUE")
        puts "--Group err"
      end
      
      #(UI.messagebox  "group Observer" )if (ENV['LDD_DEV'] == "TRUE")    
      
      LDDConstraint.constraints.removeCrashedConstraints
      
      if (ENV['LDD_DEV'] == "TRUE")
        puts "refresh end group"
      end
      
    end  
  end

  #====================================================================================

  class LDDLabelsObserverForVertex < Sketchup::EntityObserver

    @@changed = false
    
    def LDDLabelsObserverForVertex.changed=(iState)
      @@changed = iState
    end
    
    def LDDLabelsObserverForVertex.changed?
      @@changed  
    end

    def onChangeEntity(vertex)
      #puts "            VertexObserver" if (ENV['LDD_DEV'] == "TRUE")    
      @@changed = true
    end
  end

  #====================================================================================

 

  #====================================================================================

  def ldd_mode_name(mode)
    case mode
      when 51 then return "VxE_ExF"
      when 52 then return "VxE_VxF"
      when 53 then return "VxF_ExF"
      when 54 then return "VxE_VxF_ExF"
    end
    return "Unknown"
  end
  
  #===============================================================================================


  class LDDToolsObserver <  Sketchup::ToolsObserver

    @@entitiesLength = Sketchup.active_model.entities.length

    def workWithAttributes(iNumberAddedEntities)
      
      if iNumberAddedEntities <= 0
        return
      end
      puts "work with attributes!!!!!!!!!!!!!!!" if (ENV['LDD_DEV'] == "TRUE")
      (puts " iNumberAddedEntities= " + iNumberAddedEntities.to_s)if (ENV['LDD_DEV'] == "TRUE")
      entities = Sketchup.active_model.entities
      length = entities.length
      
      entitiesForDeleteAttributes = []
      
      for counter in (length - iNumberAddedEntities)..(length - 1)
        entity = entities[counter]
        
        entitiesAry = []
        
        if(entity.kind_of? Sketchup::Edge)		  
		      curve = entity.curve
      		  
		      if curve
		        if curve.kind_of? Sketchup::ArcCurve
		          entitiesAry.push curve
		        end
		      else		 
		        entitiesAry.push entity		       
		        entitiesAry.push entity.start
		        entitiesAry.push entity.end
		      end
		    end
      		
		    entitiesAry.each{ |ent|
		      restr = ent.get_attribute "LDDGUIAttributes", "Constraints", nil
      	  
		      if restr
		         entitiesForDeleteAttributes.push ent
          end		
		    }      
      end
      if entitiesForDeleteAttributes.length > 0
        Sketchup.active_model.start_operation_LDD "Work With Attributes"
        ldd_remake_attributesInSpecialEntitiesArray(entitiesForDeleteAttributes)
        Sketchup.active_model.commit_operation_LDD
      end
    end  
    

    def onActiveToolChanged(tools, toolName, toolID)    
      #(puts " toolName= " + toolName.to_s)if (ENV['LDD_DEV'] == "TRUE")
      #(puts " Ent len= " + Sketchup.active_model.entities.length.to_s)if (ENV['LDD_DEV'] == "TRUE")
      
      if((toolName == "PushPullTool")||(toolName == "MoveTool")||(toolName == "RotateTool"))      
      
        currentEntitiesLength = Sketchup.active_model.entities.length
        if(@@entitiesLength != currentEntitiesLength)                        
          self.workWithAttributes(currentEntitiesLength - @@entitiesLength)
        end
      end
      
      @@entitiesLength = Sketchup.active_model.entities.length
    end
    
    def onToolStateChanged(tools, toolName, toolID, state)    
      
      #(puts " toolName= " + toolName.to_s)if (ENV['LDD_DEV'] == "TRUE")
      #(puts " state= " + state.to_s)if (ENV['LDD_DEV'] == "TRUE")
      #(puts " Ent len= " + Sketchup.active_model.entities.length.to_s)if (ENV['LDD_DEV'] == "TRUE")
      
      if((toolName == "PushPullTool")||(toolName == "MoveTool")||(toolName == "RotateTool"))      
        if(state == 0)                
          #(puts " removing attributes ")if (ENV['LDD_DEV'] == "TRUE")
          
          
          currentEntitiesLength = Sketchup.active_model.entities.length
          
          if(@@entitiesLength != currentEntitiesLength)          
            self.workWithAttributes(currentEntitiesLength - @@entitiesLength)
          end
        
                  
          if LDDLabelsObserverForVertex.changed?          
            #LDDConstraint.constraints.recalculateLabelsPositions
            recalculateLabelsPositionsTwoTimes          
          end
        end
      end  
      
      if(state == 0)     
        @@entitiesLength = Sketchup.active_model.entities.length  
      end
      
    end
  end
   
  #===============================================================================================

  class LDDAppObserver <  Sketchup::AppObserver
   
    def onOpenModel(model)
      #puts "onOpenModel"
      model.addLDDObservers()
    end
    
    def onNewModel(model)
      #puts "onNewModel"    
      model.addLDDObservers()
    end

  end
  
end

#====================================================================================

file_loaded("ldd_common_functions.rb")