# Copyright 2008, LEDAS

# NO PART OF THIS SOFTWARE MAY BE RE-USED FOR ANY OTHER PURPOSE THAN TO RUN
# DRIVING DIMENSIONS WITHOUT THE EXPRESS AND WRITTEN APPROVAL OF LEDAS.


# YOU UNDERSTAND AND AGREE THAT LEDAS IS MAKING NO WARRANTY THAT THE SOFTWARE WILL
# MEET YOUR REQUIREMENTS, THAT THE SOFTWARE WILL BE ERROR OR BUG-FREE,
# REGARDING THE SECURITY, RELIABILITY OR PERFORMANCE OF THE SOFTWARE, OR THAT ANY
# ERRORS IN THE SOFTWARE WILL BE CORRECTED.  YOUR USE OF THE SOFTWARE IS AT YOUR SOLE RISK.
# THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND WITHOUT WARRANTY OF ANY KIND.
# TO THE EXTENT PERMITTED BY LAW, LEDAS DISCLAIMS ALL WARRANTIES AND CONDITIONS OF ANY KIND,
# WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES AND CONDITIONS
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT. ANY CONTENT 
# OR MATERIAL DOWNLOADED OR OTHERWISE OBTAINED THROUGH THE USE OF THE SOTWARE IS DONE AT
# YOUR OWN DISCRETION AND RISK AND YOU WILL BE SOLELY RESPONSIBLE FOR ANY DAMAGE TO YOUR
# COMPUTER SYSTEM OR OTHER DEVICE OR LOSS OF DATA THAT RESULTS FROM THE DOWNLOAD OF ANY
# SUCH CONTENT OR MATERIAL.  YOU CAN RECOVER FROM LEDAS AND ITS SUPPLIERS ONLY DIRECT DAMAGES
# UP TO THE AMOUNT YOU PAID FOR THE SOFTWARE.  YOU CANNOT RECOVER ANY OTHER DAMAGES, INCLUDING
# CONSEQUENTIAL, LOST PROFITS, SPECIAL, INDIRECT OR INCIDENTAL DAMAGES.
 

require 'sketchup.rb'

require "ldd\\ldd_sketchup_classes.rb"

module LDD
  #====================================================================================
  
  class LDD_constraints_types
    VERTEX_SOMETHING = 1
    
    FIXATION = 20
    EDGE_LENGTH = 21
    ARCCURVE_RADIUS = 22
    
    BINARY_CONSTRAINTS_THERESHOLD = 50
    
    EDGE_EDGE_DISTANCE = 51 
    VERTEX_EDGE_DISTANCE = 52
    EDGE_VERTEX_DISTANCE = -52
    VERTEX_VERTEX_DISTANCE = 53 
    EDGE_EDGE_PLANAR_ANGLE = 54
    EDGE_EDGE_ANGLE = 55
  end
  
  class LDD_constraints_typesVersion_0_2
    VERTEX_SOMETHING = 0
    EDGE_LENGTH = 1
    ARCCURVE_RADIUS = 2
    
    BINARY_CONSTRAINTS_THERESHOLD = 3
    
    EDGE_EDGE_DISTANCE = 3 
    VERTEX_EDGE_DISTANCE = 4
    EDGE_VERTEX_DISTANCE = -4
    VERTEX_VERTEX_DISTANCE = 5 
    EDGE_EDGE_PLANAR_ANGLE = 6
  end
  
  #====================================================================================  
  def getCorrectType(iType, iEntity1 = nil, iEntity2 = nil)
    
    if (iType == LDD_constraints_types::EDGE_EDGE_DISTANCE)
            
      if (LDDConstraint.setAngleForParallelEdges == (! LDDConstraint.switchAngleEdge))
      
        (return 0) if ((not iEntity1&&iEntity2)||(not ((iEntity1.kind_of? Sketchup::Edge)&&
                                                        (iEntity2.kind_of? Sketchup::Edge))))
        
        if(iEntity1.common_face iEntity2)
          return LDD_constraints_types::EDGE_EDGE_PLANAR_ANGLE
        else
          return LDD_constraints_types::EDGE_EDGE_ANGLE
        end        
      else
        return LDD_constraints_types::EDGE_EDGE_DISTANCE
      end
    else
      return iType    
    end
  
  end
  
  #====================================================================================

  def getConstraintType(iEntity1, iEntity2 = nil)
    
    return 0 if not iEntity1
    
    return 0 if iEntity1 == iEntity2
  
    if(iEntity2 == nil)
        
      if(iEntity1.kind_of? Sketchup::Edge)
        return LDD_constraints_types::EDGE_LENGTH
      end
      
      if(iEntity1.kind_of? Sketchup::Vertex)
        return LDD_constraints_types::VERTEX_SOMETHING
      end
      
      if(iEntity1.kind_of? Sketchup::ArcCurve)
        return LDD_constraints_types::ARCCURVE_RADIUS
      end
      
    else
      if(iEntity1.kind_of? Sketchup::Edge)
        (return 0) if (iEntity1.length == 0)
          
        if(iEntity2.kind_of? Sketchup::Edge)
          (return 0) if (iEntity2.length == 0)
        
          line_1 = iEntity1.line
          line_2 = iEntity2.line
          vector_1 = line_1[1]
          vector_2 = line_2[1]
            
          if(vector_1.parallel? vector_2)
            #if(((line_1[0]).distance_to_line line_2) > (0.0).to_l)
              return getCorrectType(LDD_constraints_types::EDGE_EDGE_DISTANCE, iEntity1, iEntity2)
            #end
          end
        
          face = iEntity1.common_face iEntity2        
          if (face != nil)            
              return LDD_constraints_types::EDGE_EDGE_PLANAR_ANGLE
          end
          
          return LDD_constraints_types::EDGE_EDGE_ANGLE          
        end      
        
        if(iEntity2.kind_of? Sketchup::Vertex)
          return LDD_constraints_types::EDGE_VERTEX_DISTANCE
        end      
      end
      
      if(iEntity1.kind_of? Sketchup::Vertex)
          
        if(iEntity2.kind_of? Sketchup::Edge)
          return LDD_constraints_types::VERTEX_EDGE_DISTANCE
        end      
        
        if(iEntity2.kind_of? Sketchup::Vertex)
          return LDD_constraints_types::VERTEX_VERTEX_DISTANCE
        end      
      end      
    end    
    
    return 0
  end

  #====================================================================================
  class LDDPreConstraint
      
    attr_accessor :entity1
    attr_accessor :entity2
    attr_accessor :drivingDimensionEntity
    
    attr_accessor :solveThenConstraintDone
    attr_accessor :type
    
    attr_accessor :commitAfterCreation
    
    attr_accessor :flags
        
    #-------------------------------------------------------------------------------------
    def initialize(iEntity, iMousePosition = nil, iOldDrivingDimensionsGroupAndTextAry = nil)     
    
      @solveThenConstraintDone = true      
      @type = 0
      
      if(iEntity)      
        @entity1 = iEntity
        @entity2 = nil
        
        if(iEntity.kind_of? Sketchup::Vertex)
          p1 = iEntity.position                          
          p2 = p1
          
          done = true
        end
      
        if(iEntity.kind_of? Sketchup::Edge)
          p1 = iEntity.start.position
          p2 = iEntity.end.position
          
          done = true
        end
        
        if(iEntity.kind_of? Sketchup::ArcCurve)          
          if iMousePosition
            p1 = iMousePosition            
          else
            p1 = iEntity.vertices[((iEntity.count_edges)/2).to_i].position		                  
          end
          
          p2 = p1
          
          done = true
        end
        
        (@drivingDimensionEntity = Driving_Dimension.new(p1, p2, nil, "Driving Dimension", iOldDrivingDimensionsGroupAndTextAry)) if done
        
        @drivingDimensionEntity.createObservers() if @drivingDimensionEntity
      else
        return nil #Error
      end
    end
    
    #-------------------------------------------------------------------------------------   
    def currentValue
      
      if @type == 0
        type_0 = getConstraintType(@entity1, @entity2)
      else
        type_0 = @type
      end
      
      case type_0
              
        when LDD_constraints_types::VERTEX_SOMETHING
          return (0.0).to_l
          
        when LDD_constraints_types::EDGE_LENGTH
          return @entity1.length
          
        when LDD_constraints_types::ARCCURVE_RADIUS
          return @entity1.radius.to_l
                    
        when LDD_constraints_types::EDGE_EDGE_DISTANCE
          return (@entity1.start.position.distance_to_line @entity2.line).to_l
          
        when LDD_constraints_types::VERTEX_EDGE_DISTANCE
          return (@entity1.position.distance_to_line @entity2.line).to_l
          
        when LDD_constraints_types::VERTEX_VERTEX_DISTANCE
          return (@entity1.position.distance @entity2.position).to_l
          
        when LDD_constraints_types::EDGE_EDGE_PLANAR_ANGLE  
          return @entity1.angleToEdge(@entity2, @drivingDimensionEntity.flags)    
          
        when LDD_constraints_types::EDGE_EDGE_ANGLE  
          return @entity1.angleToEdge(@entity2, @drivingDimensionEntity.flags)    
          
      end
        
      return (0.0).to_l
    end    
     
    #-------------------------------------------------------------------------------------
    def setCurrentPosition(iMousePosition, iSelectionEntity)
    
      if @entity2
        secondEntity = @entity2
      else
        secondEntity = iSelectionEntity      
      end
      
      if @type == 0
        type_0 = getConstraintType(@entity1, secondEntity)
      else
        type_0 = @type
      end
      
      if type_0 >= 0
        @drivingDimensionEntity.moveWithMousePosition(type_0, iMousePosition, @entity1, secondEntity)      
      else
        @drivingDimensionEntity.moveWithMousePosition(- type_0, iMousePosition, secondEntity , @entity1)            
      end
      
      
    end    
    #-------------------------------------------------------------------------------------
    
    def nextStep(iMousePosition, iSelectionEntity, iValue = nil)
    
      if not iSelectionEntity
      
        answer = []
        
        if @type == 0
          type_0 = getConstraintType(@entity1, @entity2)
        else
          type_0 = @type
        end
        
        case type_0        
          when LDD_constraints_types::VERTEX_SOMETHING
            answer[0] = LDDFixationConstraint.new(self, (0.0).to_l)
            @solveThenConstraintDone= false
            @commitAfterCreation= true
            
            dr_di = answer[0].drivingDimensionEntity
            (dr_di.vector= (dr_di.p1.vector_to dr_di.p2))if(dr_di)
            
          when LDD_constraints_types::EDGE_LENGTH
            answer[0] = LDDEdgeLengthConstraint.new(self, iValue)            
            
          when LDD_constraints_types::ARCCURVE_RADIUS
            answer[0] = LDDArcCurveRadiusConstraint.new(self, iValue)            
            
          when LDD_constraints_types::EDGE_EDGE_DISTANCE
            answer[0] = LDDEdgeEdgeDistanceConstraint.new(self, iValue)            
            
          when LDD_constraints_types::VERTEX_EDGE_DISTANCE
            answer[0] = LDDVertexEdgeDistanceConstraint.new(self, iValue)            
            
          when LDD_constraints_types::VERTEX_VERTEX_DISTANCE
            answer[0] = LDDVertexVertexDistanceConstraint.new(self, iValue)            
            
          when LDD_constraints_types::EDGE_EDGE_PLANAR_ANGLE      
            answer[0] = LDDEdgeEdgePlanarAngleConstraint.new(self, iValue)
            
          when LDD_constraints_types::EDGE_EDGE_ANGLE      
            answer[0] = LDDEdgeEdgeAngleConstraint.new(self, iValue)
            
          when LDD_constraints_types::FIXATION
            answer[0] = LDDFixationConstraint.new(self, (0.0).to_l)
            
        end
        
        LDDConstraint.switchAngleEdge= false#! LDDConstraint.switchAngleEdge
                
        if (not answer[0].drivingDimensionEntity)
          Sketchup.active_model.abort_operation_LDD
          return nil
        end
        
        answer[0].recalculateLabelPosition
        
        if @solveThenConstraintDone
          result = ldd_solve(nil)	 
        else
          Sketchup.active_model.commit_operation_LDD if @commitAfterCreation
          return nil
        end          
            
        if (result == 0)
          Sketchup.active_model.commit_operation_LDD
          recalculateLabelsPositionsTwoTimes if LDDLabelsObserverForVertex.changed? 
        else
          Sketchup.active_model.abort_operation_LDD            
        end  
        
        return nil
      else
        if getConstraintType(@entity1, iSelectionEntity) >= 0
          @entity2 = iSelectionEntity
        else
          @entity2 = @entity1
          @entity1 = iSelectionEntity          
        end
        
        return self
      end    
    end
    
    #-------------------------------------------------------------------------------------    
  end
  
  
  #====================================================================================
   
  class LDDConstraints < Array
  
    @@ldd_visible = true
  
    #-----------------------------------------------------------------------------------------------
    
    def LDDConstraints.visible? 
      return @@ldd_visible
    end
    #-----------------------------------------------------------------------------------------------
    
    def LDDConstraints.visible=(iState)
      @@ldd_visible = iState
    end
    
            
    def addConstraint(iConstraint)
      self.push iConstraint
    end
    
    def removeConstraint(iConstraint, iAddToUndoRedo = true)
      #(puts "OPERATION remove constraint" )if (ENV['LDD_DEV'] == "TRUE")
      
      (Sketchup.active_model.start_operation_LDD "Remove Constraint") if iAddToUndoRedo
      
      if(iConstraint.kind_of? Numeric)        
        constraint = self.findConstraintOnID(iConstraint)
        
        if not constraint    
          puts "Constraint Not Found" if ENV['LDD_DEV'] == "TRUE"  
          return 
        end
      elsif(iConstraint.kind_of? LDDConstraint)
        constraint = iConstraint    
      end  
      
      entity1 = constraint.entity1
      entity2 = constraint.entity2
      
      entity1.removeConstraintFromAttributes(constraint.constraintID) if entity1.valid?
                
      if (entity2 != nil)
        entity2.removeConstraintFromAttributes(constraint.constraintID) if entity2.valid?
      end
      
      self.delete constraint
      constraint.drivingDimensionEntity.erase!
      
      #puts "Constraint removed" if ENV['LDD_DEV'] == "TRUE"            
      
      Sketchup.active_model.commit_operation_LDD if iAddToUndoRedo
      
      return  
    
    end
    
    def findConstraintOnID(iID)
      return nil if(iID == 0)
    
      self.each{ |constraint|
        return constraint if (constraint.constraintID == iID)
      }
      return nil
    end
    
    def recalculateLabelsPositions(iAddToUndoRedo = true)
      #(puts "OPERATION recalculate labels" )if (ENV['LDD_DEV'] == "TRUE")
      LDDLabelsObserverForVertex.changed=(false)
      
      (Sketchup.active_model.start_operation_LDD "Recalculate Labels Positions") if iAddToUndoRedo
            
      self.removeCrashedConstraints(false)
            
      self.each { |constraint|        
        constraint.recalculateLabelPosition
      }
      
      Sketchup.active_model.commit_operation_LDD if iAddToUndoRedo      
    end  
    
    def removeCrashedConstraints(iAddToUndoRedo = true)
      #(puts "OPERATION remove Crashed" )if (ENV['LDD_DEV'] == "TRUE")
    
      to_delete = []      
      self.each { |constraint|        
        crashed = constraint.crashed?
        if(crashed)
          to_delete.push constraint
          
          if crashed.kind_of? Numeric
            if crashed == LDD_constraints_types::EDGE_EDGE_ANGLE              
          
              preConstraint = LDDPreConstraint.new(constraint.entity1)      
              preConstraint.entity2 = constraint.entity2
              preConstraint.setCurrentPosition(constraint.drivingDimensionEntity.text_entity.point, nil)
              
              preConstraint.solveThenConstraintDone = false              
              new_value = (constraint.value > Math::PI ? constraint.value - Math::PI : constraint.value)          
              preConstraint.nextStep(constraint.drivingDimensionEntity.text_entity.point, nil, new_value)
              
            end
          end
        end      
      }
      
      return if(to_delete.length == 0)
      
      (Sketchup.active_model.start_operation_LDD "Remove Crashed Constraints") if iAddToUndoRedo
      
      if (ENV['LDD_DEV'] == "TRUE")
        puts "deleted  " + to_delete.length.to_s + " restr"
      end
        
      to_delete.each{ |constraint|        
        self.delete constraint
      }        
        
      to_delete.each{ |constraint|
        #(UI.messagebox  "remove Crashed" )if (ENV['LDD_DEV'] == "TRUE")
        self.removeConstraint(constraint, false)         
      }      
      
      Sketchup.active_model.commit_operation_LDD if iAddToUndoRedo
    end      
    
    
  end 
  #====================================================================================

  class LDDConstraint
  
    FULL_TEXT = false
          
    attr_accessor :entity1
    attr_accessor :entity2
    attr_accessor :drivingDimensionEntity
    attr_accessor :value
    
    @@constraints = LDDConstraints.new
    def LDDConstraint.constraints
      @@constraints
    end
    
    def LDDConstraint.constraints=(iConstraints)
      @@constraints = iConstraints
    end
    
    @@setAngleForParallelEdges = false    
    def LDDConstraint.setAngleForParallelEdges
      @@setAngleForParallelEdges
    end    
    def LDDConstraint.setAngleForParallelEdges=(iState)
      @@setAngleForParallelEdges = iState
    end
    
    @@switchAngleEdge = false    
    def LDDConstraint.switchAngleEdge
      @@switchAngleEdge
    end    
    def LDDConstraint.switchAngleEdge=(iState)
      @@switchAngleEdge = iState
    end
    
    @@immediatelySolve = true
    def LDDConstraint.immediatelySolve
      @@immediatelySolve
    end    
    def LDDConstraint.immediatelySolve=(iState)
      @@immediatelySolve = iState
    end
    
    
    
    def initialize(iPreConstraint, iValue, iConstraintType = nil)
      
      return false if not iValue
    
      @entity1 = iPreConstraint.entity1
      @entity2 = iPreConstraint.entity2
      
      @drivingDimensionEntity = iPreConstraint.drivingDimensionEntity      
      @value = iValue
      
      if not iConstraintType
        iConstraintType = getConstraintType(@entity1, @entity2)
      end
      
      fullRestr = @entity1.addLDDConstraintToAttributes([iConstraintType, @value, self.constraintID(), 1], @entity2)
      
      if @entity2
        fullRestr = @entity2.addLDDConstraintToAttributes([iConstraintType, @value, self.constraintID(), 2], @entity1)
      end
            
      @@constraints.removeConstraint(fullRestr, false) if fullRestr
      
      @entity1.createObservers()
      @entity2.createObservers() if @entity2      
      
      @@constraints.addConstraint(self)
      
      return iValue      
    end

    def change!(iValue)
      if iValue
        @value = iValue
        
        self.entities.each{ |entity| 
      
          constraints = entity.get_attribute "LDDGUIAttributes", "Constraints", nil

          constraint = entity.getAttributeConstraintOnID(self.constraintID)    

          index = constraints.index(constraint) if constraint   
          (constraints[index])[1] = iValue if constraint
      
          entity.set_attribute "LDDGUIAttributes", "Constraints", constraints
        }
        return iValue
      else
        return false
      end
    end
    
    def recalculateLabelPosition      
      if (self.value.to_l == self.currentValue.to_l)
        @drivingDimensionEntity.material= 32768 #"Green"
      else
        @drivingDimensionEntity.material= 255 #"Red"
      end
      
      return true  
    end
    
    
    def == (iLDDConstraint)
      return false if not iLDDConstraint
      
      return false if not iLDDConstraint.kind_of? LDDConstraint
      
      return (self.constraintID == iLDDConstraint.constraintID)
    end
    
    def constraintID
      return 0 if not @drivingDimensionEntity
      
      return @drivingDimensionEntity.getID
    end
    
    def entities
      ary = [@entity1]
      (ary.push @entity2) if @entity2
      return ary     
    end
    
    def currentValue
      (0.0).to_l
    end
    
    def crashed?
      return ((self.drivingDimensionEntity.group.deleted?)||
              (self.drivingDimensionEntity.text_entity.deleted?)||
              (@entity1.deleted?)||
              (@entity2 && @entity2.deleted?)||
              (@entity1 == @entity2))
    end
  end

    #====================================================================================
  
  class LDDEdgeLengthConstraint < LDDConstraint   
  
    def initialize(iPreConstraint, iValue = nil)
      if not iValue      
        result = UI.inputbox ["Length"], [iPreConstraint.currentValue.to_s], "Edge Length"
        answer = checkTypedValue(result)
      else
        answer = iValue
      end
      
      if answer        
        return super(iPreConstraint, answer, LDD_constraints_types::EDGE_LENGTH)
      else
        return false
      end
    end

    def change!(iValue = nil)

      if not iValue
        iValue = UI.inputbox ["Length"], [self.currentValue.to_s], "Edge Length"
        iValue = checkTypedValue(iValue)
      end          
            
      return super(iValue) && recalculateLabelPosition
    end
    
    def checkTypedValue(iValueInStringInArray)
      return false if not iValueInStringInArray
      
      result = iValueInStringInArray[0].to_l_LDD      
      
      if(result)
        return result
      else
        ldd_error_messagebox(LDD_return_values::LDD_TYPED_INCORRECT_VALUE)                
        return false
      end
    end
    
    def recalculateLabelPosition
      @drivingDimensionEntity.moveAutoForEdge(@entity1)
      @drivingDimensionEntity.text= (FULL_TEXT ? "EDGE LENGTH= " : "") + @value.to_s
      
      super
    end
    
    def type
      return LDD_constraints_types::EDGE_LENGTH
    end
    
    def name
      return "EDGE LENGTH"
    end
    
    def currentValue
      return @entity1.length
    end
    
    def crashed?
      return super||(self.currentValue == 0)
    end
  end
    
  #====================================================================================

  class LDDArcCurveRadiusConstraint < LDDConstraint   
  
    def initialize(iPreConstraint, iValue = nil)
      @entity1 = iPreConstraint.entity1
      @entity2 = iPreConstraint.entity2
      
      if not iValue
        result = UI.inputbox ["Radius"], [iPreConstraint.currentValue.to_s], "ArcCurve Radius"
        answer = checkTypedValue(result)
      else
        answer = iValue
      end
      
      if answer
        return super(iPreConstraint, answer, LDD_constraints_types::ARCCURVE_RADIUS)
      else
        return false   
      end
    end

    def change!(iValue = nil)

      if not iValue
        iValue = UI.inputbox ["Radius"], [self.currentValue.to_s], "ArcCurve Radius"
        iValue = checkTypedValue(iValue)
      end                    
      
      return super(iValue) && recalculateLabelPosition
    end
    
    def checkTypedValue(iValueInStringInArray)
      return false if not iValueInStringInArray
      
      result = iValueInStringInArray[0].to_l_LDD      
      
      if(result)        
        start_v = @entity1.first_edge.start 
        end_v = @entity1.last_edge.end
      
        if (start_v == end_v)# circle
          return result
        end
        # check if value correct for ARC
      
        vector = start_v.position - end_v.position
        length = vector.length
      
        if (length > 2*result)
          ldd_error_messagebox(LDD_return_values::LDD_TYPED_INCORRECT_VALUE)                
          return false
        end  
        
        return result        
      else
        ldd_error_messagebox(LDD_return_values::LDD_TYPED_INCORRECT_VALUE)                
        return false
      end
    end
    
    def recalculateLabelPosition
      @drivingDimensionEntity.moveAutoForArcCurve(@entity1)
      @drivingDimensionEntity.text= (FULL_TEXT ? "ARCCURVE RADIUS= " : "") + @value.to_s
      
      super
    end
    
    def type
      return LDD_constraints_types::ARCCURVE_RADIUS
    end
    
    def name
      return "ARCCURVE RADIUS"
    end
    
    def currentValue
      return @entity1.radius.to_l
    end
    
    def crashed?
      return super||(self.currentValue == 0)
    end
    
  end


  #====================================================================================

  class LDDEdgeEdgeDistanceConstraint < LDDConstraint   
  
    def initialize(iPreConstraint, iValue = nil)      
      if not iValue
        result = UI.inputbox ["Distance"], [iPreConstraint.currentValue.to_s], "Edge-Edge Distance"
        answer = checkTypedValue(result)
      else
        answer = iValue
      end
      
      if answer
        return super(iPreConstraint, answer, LDD_constraints_types::EDGE_EDGE_DISTANCE)
      else
        return false   
      end
    end

    def change!(iValue = nil)

      if not iValue
        iValue = UI.inputbox ["Distance"], [self.currentValue.to_s], "Edge-Edge Distance"
        iValue = checkTypedValue(iValue)
      end                    
            
      return super(iValue) && recalculateLabelPosition
    end
    
    def checkTypedValue(iValueInStringInArray)
      return false if not iValueInStringInArray
      
      result = iValueInStringInArray[0].to_l_LDD      
      
      if(result)
        return result
      else
        ldd_error_messagebox(LDD_return_values::LDD_TYPED_INCORRECT_VALUE)                
        return false
      end
    end
    
    def recalculateLabelPosition
      @drivingDimensionEntity.moveAutoForTwoParallelEdges(@entity1, @entity2)
      @drivingDimensionEntity.text= (FULL_TEXT ? "EDGE-EDGE DISTANCE= " : "") + @value.to_s
      
      super
    end
    
    def type
      return LDD_constraints_types::EDGE_EDGE_DISTANCE
    end
    
    def name
      return "EDGE-EDGE DISTANCE"
    end
    
    def currentValue
      return  (@entity1.start.position.distance_to_line @entity2.line).to_l
    end
    
    def crashed?
      return super||(@entity1.length == 0)||(@entity2.length == 0)
    end
  end
  #====================================================================================

  class LDDVertexEdgeDistanceConstraint < LDDConstraint     
    def initialize(iPreConstraint, iValue = nil)
      if not iValue
        result = UI.inputbox ["Distance"], [iPreConstraint.currentValue.to_s], "Vertex-Edge Distance"
        answer = checkTypedValue(result)
      else
        answer = iValue
      end
      
      if answer
        return super(iPreConstraint, answer, LDD_constraints_types::VERTEX_EDGE_DISTANCE)
      else
        return false   
      end
    end
    
    def change!(iValue = nil)

      if not iValue
        iValue = UI.inputbox ["Distance"], [self.currentValue.to_s], "Vertex-Edge Distance"
        iValue = checkTypedValue(iValue)
      end                    
            
      return super(iValue) && recalculateLabelPosition
    end
    
    def checkTypedValue(iValueInStringInArray)
      return false if not iValueInStringInArray
      
      result = iValueInStringInArray[0].to_l_LDD      
      
      if(result)
        return result
      else
        ldd_error_messagebox(LDD_return_values::LDD_TYPED_INCORRECT_VALUE)                
        return false
      end
    end
    
    def recalculateLabelPosition
      @drivingDimensionEntity.moveAutoForVertexAndEdge(@entity1, @entity2)
      @drivingDimensionEntity.text= (FULL_TEXT ? "VERTEX-EDGE DISTANCE= " : "") + @value.to_s
      
      super
    end

    def type
      return LDD_constraints_types::VERTEX_EDGE_DISTANCE
    end
    
    def name
      return "VERTEX-EDGE DISTANCE"
    end
    
    def currentValue
      return  (@entity1.position.distance_to_line @entity2.line).to_l
    end
    
    def crashed?
      return super||(@entity2.length == 0)
    end
  end
  #====================================================================================


  class LDDVertexVertexDistanceConstraint < LDDConstraint   
  
    def initialize(iPreConstraint, iValue = nil)
      if not iValue
        result = UI.inputbox ["Distance"], [iPreConstraint.currentValue.to_s], "Vertex-Vertex Distance"
        answer = checkTypedValue(result)
      else
        answer = iValue
      end
      
      if answer
        return super(iPreConstraint, answer, LDD_constraints_types::VERTEX_VERTEX_DISTANCE)
      else
        return false   
      end
    end
    
    def change!(iValue = nil)

      if not iValue
        iValue = UI.inputbox ["Distance"], [self.currentValue.to_s], "Vertex-Vertex Distance"
        iValue = checkTypedValue(iValue)
      end
      
      return super(iValue) && recalculateLabelPosition
    end
    
    def checkTypedValue(iValueInStringInArray)
      return false if not iValueInStringInArray
      
      result = iValueInStringInArray[0].to_l_LDD      
      
      if(result)
        return result
      else
        ldd_error_messagebox(LDD_return_values::LDD_TYPED_INCORRECT_VALUE)                
        return false
      end
    end
    
    def recalculateLabelPosition
      @drivingDimensionEntity.moveAutoForTwoVertices(@entity1, @entity2)
      @drivingDimensionEntity.text= (FULL_TEXT ? "VERTEX-VERTEX DISTANCE= " : "") + @value.to_s
      
      super
    end
    
    def type
      return LDD_constraints_types::VERTEX_VERTEX_DISTANCE
    end
    
    def name
      return "VERTEX-VERTEX DISTANCE"
    end
    
    def currentValue
      return  (@entity1.position.distance @entity2.position).to_l
    end
    
    def crashed?
      return super||(self.currentValue == 0)
    end
  end
  #====================================================================================

  class LDDEdgeEdgePlanarAngleConstraint < LDDConstraint
    def initialize(iPreConstraint, iValue = nil)
      if not iValue
        value_new = ((RAD2DEG(iPreConstraint.currentValue) * 1000).round.to_f)/1000
        result = UI.inputbox ["Angle in Degrees"], [value_new.to_s], "Edge-Edge Angle"      
        answer = checkTypedValue(result)        
        answer = DEG2RAD(answer) if answer
      else
        answer = iValue
      end
      
      if answer
        return super(iPreConstraint, answer, LDD_constraints_types::EDGE_EDGE_PLANAR_ANGLE)
      else
        return false   
      end
    end

    def change!(iValue = nil)

      if not iValue        
        value_new = ((RAD2DEG(self.currentValue) * 1000).round.to_f)/1000
        iValue = UI.inputbox ["Angle in Degrees"], [value_new.to_s], "Edge-Edge Angle"      
        iValue = checkTypedValue(iValue)
        iValue = DEG2RAD(iValue) if iValue
      end
      
      return super(iValue) && recalculateLabelPosition
    end
    
    def checkTypedValue(iValueInStringInArray)
      return false if not iValueInStringInArray
      
      result = iValueInStringInArray[0].to_f_LDD      
      
      if(result && (0.0 <= result)&&(result <= 360.0))
        return result
      else
        ldd_error_messagebox(LDD_return_values::LDD_TYPED_INCORRECT_VALUE)                
        return false
      end
    end
    
    def recalculateLabelPosition      
      @drivingDimensionEntity.moveAutoForTwoGeneralCaseEdges(@entity1, @entity2)
      @drivingDimensionEntity.text= (FULL_TEXT ? "EDGE-EDGE ANGLE= " : "") + RAD2DEG(@value).to_s + ''.to_s
      
      super
    end
    
    def type
      return LDD_constraints_types::EDGE_EDGE_PLANAR_ANGLE
    end
    
    def name
      return "EDGE-EDGE ANGLE"
    end
    
    def currentValue
      return @entity1.angleToEdge(@entity2, @drivingDimensionEntity.flags)
    end
    
    def crashed?
      state = (super||(@entity1.length == 0)||(@entity2.length == 0))
      
      if state
        return true
      else
        if(@entity1.common_face @entity2)
          return false
        else
          return LDD_constraints_types::EDGE_EDGE_ANGLE
        end
      end
    end
    
  end
  #====================================================================================
  
  class LDDEdgeEdgeAngleConstraint < LDDConstraint
    def initialize(iPreConstraint, iValue = nil)
      if not iValue
        value_new = ((RAD2DEG(iPreConstraint.currentValue) * 1000).round.to_f)/1000
        result = UI.inputbox ["Angle in Degrees"], [value_new.to_s], "Edge-Edge Angle"      
        answer = checkTypedValue(result)        
        answer = DEG2RAD(answer) if answer
      else
        answer = iValue
      end
      
      if answer
        return super(iPreConstraint, answer, LDD_constraints_types::EDGE_EDGE_ANGLE)
      else
        return false   
      end
    end

    def change!(iValue = nil)

      if not iValue        
        value_new = ((RAD2DEG(self.currentValue) * 1000).round.to_f)/1000
        iValue = UI.inputbox ["Angle in Degrees"], [value_new.to_s], "Edge-Edge Angle"      
        iValue = checkTypedValue(iValue)
        iValue = DEG2RAD(iValue) if iValue
      end
      
      return super(iValue) && recalculateLabelPosition
    end
    
    def checkTypedValue(iValueInStringInArray)
      return false if not iValueInStringInArray
      
      result = iValueInStringInArray[0].to_f_LDD      
      
      if(result && (0.0 <= result)&&(result <= 180.0))
        return result
      else
        ldd_error_messagebox(LDD_return_values::LDD_TYPED_INCORRECT_VALUE)                
        return false
      end
    end
    
    def recalculateLabelPosition      
      @drivingDimensionEntity.moveAutoForTwoGeneralCaseEdges(@entity1, @entity2)
      @drivingDimensionEntity.text= (FULL_TEXT ? "EDGE-EDGE ANGLE= " : "") + RAD2DEG(@value).to_s + ''.to_s
      
      super
    end
    
    def type
      return LDD_constraints_types::EDGE_EDGE_ANGLE
    end
    
    def name
      return "EDGE-EDGE ANGLE"
    end
    
    def currentValue
      return @entity1.angleToEdge(@entity2, @drivingDimensionEntity.flags)
    end
    
    def crashed?
      return super||(@entity1.length == 0)||(@entity2.length == 0)
    end
  end
  #====================================================================================
  
  class LDDFixationConstraint < LDDConstraint   
  
    def initialize(iPreConstraint, iValue = (0.0).to_l)      
      return super(iPreConstraint, iValue, LDD_constraints_types::FIXATION)      
    end
    
    def change!(iValue = nil)
      return (0.0).to_l
    end
    
    def recalculateLabelPosition
      @drivingDimensionEntity.moveAutoForFixationConstraint(@entity1)
      @drivingDimensionEntity.text= "FIXED " + @entity1.typename.upcase
      
      @drivingDimensionEntity.material= 32768 #"Green"      
    end
    
    def type
      return LDD_constraints_types::FIXATION
    end
    
    def name
      return "FIXATION"
    end
    
    def crashed?
      return super
    end
  end
  
  #====================================================================================
end

file_loaded("ldd_constraints.rb")