# Copyright 2008, LEDAS

# NO PART OF THIS SOFTWARE MAY BE RE-USED FOR ANY OTHER PURPOSE THAN TO RUN
# DRIVING DIMENSIONS WITHOUT THE EXPRESS AND WRITTEN APPROVAL OF LEDAS.


# YOU UNDERSTAND AND AGREE THAT LEDAS IS MAKING NO WARRANTY THAT THE SOFTWARE WILL
# MEET YOUR REQUIREMENTS, THAT THE SOFTWARE WILL BE ERROR OR BUG-FREE,
# REGARDING THE SECURITY, RELIABILITY OR PERFORMANCE OF THE SOFTWARE, OR THAT ANY
# ERRORS IN THE SOFTWARE WILL BE CORRECTED.  YOUR USE OF THE SOFTWARE IS AT YOUR SOLE RISK.
# THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND WITHOUT WARRANTY OF ANY KIND.
# TO THE EXTENT PERMITTED BY LAW, LEDAS DISCLAIMS ALL WARRANTIES AND CONDITIONS OF ANY KIND,
# WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES AND CONDITIONS
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT. ANY CONTENT 
# OR MATERIAL DOWNLOADED OR OTHERWISE OBTAINED THROUGH THE USE OF THE SOTWARE IS DONE AT
# YOUR OWN DISCRETION AND RISK AND YOU WILL BE SOLELY RESPONSIBLE FOR ANY DAMAGE TO YOUR
# COMPUTER SYSTEM OR OTHER DEVICE OR LOSS OF DATA THAT RESULTS FROM THE DOWNLOAD OF ANY
# SUCH CONTENT OR MATERIAL.  YOU CAN RECOVER FROM LEDAS AND ITS SUPPLIERS ONLY DIRECT DAMAGES
# UP TO THE AMOUNT YOU PAID FOR THE SOFTWARE.  YOU CANNOT RECOVER ANY OTHER DAMAGES, INCLUDING
# CONSEQUENTIAL, LOST PROFITS, SPECIAL, INDIRECT OR INCIDENTAL DAMAGES.

 
require 'sketchup.rb'


require "ldd\\ldd_class.rb"
require "ldd\\ldd_constraints.rb"
require "ldd\\ldd_common_functions.rb"



module LDD 
  class Driving_Dimensions_Fixation_Tool
    @@ldd_fixation_tool_choosen = -1
    
    def Driving_Dimensions_Fixation_Tool.active?
      return @@ldd_fixation_tool_choosen
    end
      
    def activate
      @@ldd_fixation_tool_choosen = 1
    end
    
    def deactivate view
      @@ldd_fixation_tool_choosen = -1
      self.reset      
    end
    
    def suspend view 
      @@ldd_fixation_tool_choosen = 0
    end
    
    def resume view 
      @@ldd_fixation_tool_choosen = 1
    end
    
    def initialize      
      @ip = Sketchup::InputPoint.new
      @ip2 = Sketchup::InputPoint.new
      
      @operatedConstraint = nil
    end
    #-----------------------------------------------------------------------------------------------
    def draw(view)
      @ip.draw(view)
    end
    #-----------------------------------------------------------------------------------------------	
    
    def reset          
      @operatedConstraint = nil      
      
      Sketchup.active_model.abort_operation_LDD	
    end
       
    #-----------------------------------------------------------------------------------------------
    def onLButtonDown(flags, x, y, view)  
      @ip2.pick view, x, y      
      
      if @operatedConstraint
      
        if @operatedConstraint.kind_of? LDDPreConstraint
          @operatedConstraint = @operatedConstraint.nextStep(@ip2.position, nil)
        else
          #ERROR
        end
      
      else # not @operatedConstraint
        selection = selection_Fixation(view, x, y)
        
        if selection
          if selection.kind_of? LDDConstraint #Move label           
            Sketchup.active_model.start_operation_LDD "Move Driving Dimensions"
            @operatedConstraint = selection
            return
          end
          
          if selection.kind_of? Sketchup::Entity
            Sketchup.active_model.start_operation_LDD "Driving Dimensions Fixation"
            @operatedConstraint = LDDPreConstraint.new(selection,@ip2.position)
            
            @operatedConstraint.solveThenConstraintDone= false
            @operatedConstraint.commitAfterCreation= true
            @operatedConstraint.type= LDD_constraints_types::FIXATION
          end
          
        end      
      end      
    end
    #-----------------------------------------------------------------------------------------------
    def onLButtonUp(flags, x, y, view)
      @ip.pick view, x, y
      view.invalidate
            
      if @operatedConstraint      
        if @operatedConstraint.kind_of? LDDConstraint 
           @operatedConstraint = nil          
           Sketchup.active_model.commit_operation_LDD
           recalculateLabelsPositionsTwoTimes         
          return
        end
        
        if @operatedConstraint.kind_of? LDDPreConstraint          
          if((@ip.position.distance @ip2.position) > (1.0).to_l )
            @operatedConstraint = @operatedConstraint.nextStep(@ip.position, nil)
          end
          return
        end        
      end      
    end
    
    #-----------------------------------------------------------------------------------------------
    
    
    def onCancel(flag, view)
        view.invalidate 
        self.reset
    end
    #-----------------------------------------------------------------------------------------------
    
    def onLButtonDoubleClick(flags, x, y, view)
    end    
    #-----------------------------------------------------------------------------------------------
    
    def onMouseMove (flags, x, y, view)
      @ip.pick view, x, y
      view.invalidate
      
      if @operatedConstraint
      
        if @operatedConstraint.kind_of? LDDConstraint                    
          @operatedConstraint.drivingDimensionEntity.moveWithMousePosition(@operatedConstraint.type, 
                                                                            @ip.position,
                                                                            @operatedConstraint.entity1,
                                                                            @operatedConstraint.entity2)
          return
        end
          	      
        if @operatedConstraint.kind_of? LDDPreConstraint          
          @operatedConstraint.setCurrentPosition(@ip.position, nil)
          return
        end
      else
        selection_Fixation(view, x, y)      
      end            
      
    end
    
    def selection_Fixation(view, x, y)             
      sss = Sketchup.active_model.selection
      sss.clear
            
      entity = ldd_pick(view, x, y)      
      return nil if not entity
      
      if(entity.kind_of? Sketchup::Text)
        return textEntityToSelection(view, entity, sss)      
      end
      
      if not entity.kind_of? Sketchup::Vertex
        if entity.kind_of? Sketchup::ArcCurve
          sss.add entity.edges
        else
          sss.add entity
        end
      end
            
      view.tooltip = "Fix " + entity.typename
      view.invalidate
    
     return entity
    end      
  end  
end

file_loaded("ldd_fixation_tool.rb")