# Copyright 2008, LEDAS

# NO PART OF THIS SOFTWARE MAY BE RE-USED FOR ANY OTHER PURPOSE THAN TO RUN
# DRIVING DIMENSIONS WITHOUT THE EXPRESS AND WRITTEN APPROVAL OF LEDAS.


# YOU UNDERSTAND AND AGREE THAT LEDAS IS MAKING NO WARRANTY THAT THE SOFTWARE WILL
# MEET YOUR REQUIREMENTS, THAT THE SOFTWARE WILL BE ERROR OR BUG-FREE,
# REGARDING THE SECURITY, RELIABILITY OR PERFORMANCE OF THE SOFTWARE, OR THAT ANY
# ERRORS IN THE SOFTWARE WILL BE CORRECTED.  YOUR USE OF THE SOFTWARE IS AT YOUR SOLE RISK.
# THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND WITHOUT WARRANTY OF ANY KIND.
# TO THE EXTENT PERMITTED BY LAW, LEDAS DISCLAIMS ALL WARRANTIES AND CONDITIONS OF ANY KIND,
# WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES AND CONDITIONS
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT. ANY CONTENT 
# OR MATERIAL DOWNLOADED OR OTHERWISE OBTAINED THROUGH THE USE OF THE SOTWARE IS DONE AT
# YOUR OWN DISCRETION AND RISK AND YOU WILL BE SOLELY RESPONSIBLE FOR ANY DAMAGE TO YOUR
# COMPUTER SYSTEM OR OTHER DEVICE OR LOSS OF DATA THAT RESULTS FROM THE DOWNLOAD OF ANY
# SUCH CONTENT OR MATERIAL.  YOU CAN RECOVER FROM LEDAS AND ITS SUPPLIERS ONLY DIRECT DAMAGES
# UP TO THE AMOUNT YOU PAID FOR THE SOFTWARE.  YOU CANNOT RECOVER ANY OTHER DAMAGES, INCLUDING
# CONSEQUENTIAL, LOST PROFITS, SPECIAL, INDIRECT OR INCIDENTAL DAMAGES.


require 'sketchup.rb'

#====================================================================================

class Sketchup::Entities

  def eraseEntitiesLDD(*iEntitiesArray)
    iEntitiesArray.flatten!
    iEntitiesArray.uniq!
    
    iEntitiesArray.each{ |entity|
      ((self.erase_entities entity) if entity.valid?) if entity
    }
      
  end
end

#====================================================================================

$opreationStack = []

class Sketchup::Model
  def addLDDObservers    
    self.tools.add_observer(LDDToolsObserver.new)
    ldd_remake_labels(self) 
    self.set_attribute "LDD_VERSION", "LDD_VERSION", LEDAS_DRIVING_DIMENSIONS_VERSION
  end
  
  def start_operation_LDD(iOperationName, iFlag = true)    
    #if (ENV['LDD_DEV'] == "TRUE")
    #  $opreationStack.push iOperationName
    #  
    # puts "\n ===== Start Operation"
    #  puts iOperationName
    #  
    #  puts "stack after"
    #  puts "     -------"
    #  
    #  $opreationStack.each{ |a|
    #    puts "     "+a
    #  }
    #  puts "     -------"
    #  
    #end
    
    if Sketchup.version[0,1].to_i >= 7
      self.start_operation(iOperationName, iFlag)
    else
      self.start_operation(iOperationName)
    end
  end
  
  def commit_operation_LDD      
    self.commit_operation
  end
  
  def abort_operation_LDD    
    self.abort_operation
  end
end


#====================================================================================
class Sketchup::ArcCurve

  def createObservers
    self.add_observer(LDDDrDiEntityObserver.new)
      
    self.first_edge.start.add_observer(LDDLabelsObserverForVertex.new)
    self.last_edge.end.add_observer(LDDLabelsObserverForVertex.new)
    self.vertices[((self.count_edges)/2).to_i].add_observer(LDDLabelsObserverForVertex.new)
  end
  
  def getArcCurveToCylinder    
  
    vertices = self.vertices
    
    vertices.each{ |vertex|
    
      edges = vertex.edges
      
      edges.each{ |edge|
        if((edge.smooth?)||(edge.soft?)) #on cylinder edge
          anotherVertex = edge.other_vertex vertex
          
          anotherEdges = anotherVertex.edges
          
          anotherEdges.each{ |anotherEdge|
            curve = anotherEdge.curve
            
            if curve
              if curve != self            
                if(curve.kind_of? Sketchup::ArcCurve)
                  if(self.normal.parallel? curve.normal)
                    if((self.radius - curve.radius) < 0.1)
                      return curve
                    end
                  end
                end
              end
            end
          }          
        end
      }  
    }
    
    return nil      
  end  
end
#====================================================================================
class Geom::Point3d
  def between?(iPoint1, iPoint2)
    vec1 = self.vector_to iPoint1
    vec2 = self.vector_to iPoint2
    return ((vec1.length == 0)||(vec2.length == 0)||(not vec1.samedirection? vec2))
  end
  
  def flagForEdgeInAngularConstraint(iStartPointOfEdge, iEndPointOfEdge)
  # self -generalPoint
            
    if (self.distance iStartPointOfEdge) > (self.distance iEndPointOfEdge)
      return ((iStartPointOfEdge.vector_to iEndPointOfEdge).samedirection? (self.vector_to iStartPointOfEdge))
    else
      return ((iStartPointOfEdge.vector_to iEndPointOfEdge).samedirection? (self.vector_to iEndPointOfEdge))
    end
    
  end
end

#====================================================================================

class Geom::Vector3d
  def between?(iVector1, iVector2, iNormal)
    #xaxis - iVector1
    
    angleBound = iVector1.getPlanarAngleForVector(iVector2, iNormal) #angle (1-self)
    
    angleSelf = iVector1.getPlanarAngleForVector(self, iNormal) #angle (1-2)
    
    return (angleSelf < angleBound)
  end
  
  def getPlanarAngleForVector(iVector2, iNormal)
    #xaxis = self
  
    yaxis = iNormal * self

    x_angle = self.angle_between iVector2
    y_angle = yaxis.angle_between iVector2
  
    pi_dev2 = (Math::PI)/2
                
    if(x_angle <= pi_dev2)
      if(y_angle <= pi_dev2)
        #x_angle <= pi/2  
        #y_angle <= pi/2
                    
        return x_angle                 
      else
        #x_angle <= pi/2  
        #y_angle > pi/2
                    
        return (pi_dev2*4 - x_angle)                  
      end
    else
      if(y_angle <= pi_dev2)
        #x_angle > pi/2  
        #y_angle <= pi/2
                    
        return x_angle                  
      else
        #x_angle > pi/2  
        #y_angle > pi/2
                    
        return (pi_dev2*4 - x_angle)                  
      end
    end
  end
end

#====================================================================================
class Sketchup::Entity
  def removeConstraintFromAttributes(iConstraintID)
    constraints = self.get_attribute "LDDGUIAttributes", "Constraints", nil
    
    constraint = self.getAttributeConstraintOnID(iConstraintID)
    
    constraints.delete(constraint) if(constraint && constraints)
    
    self.set_attribute "LDDGUIAttributes", "Constraints", constraints
    
  end
  
  def getAttributeConstraintOnID(iConstraintID)
    constraints = self.get_attribute "LDDGUIAttributes", "Constraints", nil
    
    if constraints    
    
      constraints.each { |constraint|
    
        if(iConstraintID == constraint[2])          
          
          return constraint 
        end  
      }   
      
      return nil
    else
      #Error
      return nil
    end
  end
  
  def addLDDConstraintToAttributes(iConstraintToAttribute, iSecondEntity = nil)
    constraints = self.get_attribute "LDDGUIAttributes", "Constraints", nil
    
    to_return = [nil]
    
    if (not constraints)
      self.set_attribute "LDDGUIAttributes", "Constraints", [iConstraintToAttribute]
      return nil
    else      
      (puts "restr.length = " + constraints.length.to_s) if (ENV['LDD_DEV'] == "TRUE")
      
      constraints.each{ |constraint|
      
        if(constraint[0] == iConstraintToAttribute[0])   #equal types of constraint
        
        
          if(constraint[0] >= LDD_constraints_types::BINARY_CONSTRAINTS_THERESHOLD)  # Binary constraint:)
          
            fullConstraint = LDDConstraint.constraints.findConstraintOnID(constraint[2])    #find full constraint to get second entity
            
            if(fullConstraint)
                          
              if((fullConstraint.entity1 == self)&&(fullConstraint.entity2 == iSecondEntity)||          #it's constraint like our one
                          (fullConstraint.entity2 == self)&&(fullConstraint.entity1 == iSecondEntity))
                        
                if(iConstraintToAttribute[2] != constraint[2]) #differ IDs #so not need now
                
                  constraints.delete(constraint)
                                    
                  to_return[0] = fullConstraint
                end
              end
            end
          else  #Monadic constraint
            if(iConstraintToAttribute[2] != constraint[2])   #differ IDs #so not need now
            
              constraints.delete(constraint)
              
              to_return[0] = constraint[2]
            end
          end
        end
      }       
      
      constraints.push iConstraintToAttribute 
      
      self.set_attribute "LDDGUIAttributes", "Constraints", constraints
      puts "set atr"
      return to_return[0]
    end
  end
end
#====================================================================================

class Sketchup::Edge

  def commonPlane(iEdge, iFlags = [true, true, true])
    normal = self.normalTheirSelfTwoEdges(iEdge, iFlags)
    
    return [self.start.position, normal]
  end
  
  def getIntersectionPoint(iEdge2)
    
    selfLine = self.line    
    edge2Line = iEdge2.line    
    
    point0 = Geom.intersect_line_line selfLine, edge2Line
    
    return point0 if point0    
    return nil if((edge2Line[0].distance_to_line selfLine) > 0)
    
    if(edge2Line[1].samedirection? selfLine[1])    
      return Geom.linear_combination 0.5, self.start.position, 0.5, iEdge2.end.position
    else
      return Geom.linear_combination 0.5, self.start.position, 0.5, iEdge2.start.position
    end
    
    return nil
  end
  
  def linesType(iEdge2)    
  
    point0 = self.getIntersectionPoint(iEdge2)      
    if point0
      face = self.common_face iEdge2
      
      if face
        return INTERSECTING_LINES_WITH_COMMON_FACE
      else        
        return INTERSECTING_LINES
      end
    else
      e1Line = self.line      
      e2Line = iEdge2.line
      
      if(e1Line[1].parallel? e2Line[1])
        return PARALLEL_LINES
      else
        return GENERAL_CASE_LINES
      end      
    end
    
    return GENERAL_CASE_LINES
    
  end

  def getFlagsForTwoGeneralCaseEdgesConstraint(iOldFlags, iEdge2, iMousePosition)
    #returns [internAngleOrNot, ChangeEdgeSelfVectorDirection, ChangeEdge2VectorDirection]
    
    # default nil equals true
    
    # change internAngleOrNot just if lines are crossed
    
    # default values [nil, nil, nil]
        
    # if lines are parallel =>  if internAngleOrNot == nil =>
    # set ChangeEdge1VectorDirection = true and ChangeEdge2VectorDirection = true
    # if vectors have same direction and ChangeEdge2VectorDirection = false in another way
    #                           if internAngleOrNot != nil => not changes anything
  
    case self.linesType(iEdge2)      
    
      when PARALLEL_LINES
        if iOldFlags[0] == nil
      
          iOldFlags[0] = true
          iOldFlags[1] = true
          iOldFlags[2] = (self.line[1].samedirection? iEdge2.line[1])
        end
      
      when INTERSECTING_LINES
        point0 = self.getIntersectionPoint(iEdge2)
        
        #if iOldFlags[0] == nil                    
          iOldFlags[1] = point0.flagForEdgeInAngularConstraint(self.start.position, self.end.position)
          iOldFlags[2] = point0.flagForEdgeInAngularConstraint(iEdge2.start.position, iEdge2.end.position)          
        #end
        
        iOldFlags[0] = true
        
      when INTERSECTING_LINES_WITH_COMMON_FACE
        point0 = self.getIntersectionPoint(iEdge2)
        
        #if iOldFlags[0] == nil    
          iOldFlags[1] = point0.flagForEdgeInAngularConstraint(self.start.position, self.end.position)
          iOldFlags[2] = point0.flagForEdgeInAngularConstraint(iEdge2.start.position, iEdge2.end.position)              
        #end
        
        face = self.common_face iEdge2      
      
        normal = face.normal
        mouseVector = point0.vector_to iMousePosition
        edge1Vector = self.vectorLDD(iOldFlags[1])
        edge2Vector = iEdge2.vectorLDD(iOldFlags[2])
        
        iOldFlags[0] = mouseVector.between?(edge1Vector, edge2Vector, normal)
        
      when GENERAL_CASE_LINES
        plane = self.commonPlane(iEdge2, iOldFlags)
        
        e2StartOnPlane = iEdge2.start.position.project_to_plane plane
        e2EndOnPlane = iEdge2.end.position.project_to_plane plane
        
        point0 = Geom.intersect_line_line self.line, [e2StartOnPlane, e2EndOnPlane]
        
        iOldFlags[1] = point0.flagForEdgeInAngularConstraint(self.start.position, self.end.position)
        iOldFlags[2] = point0.flagForEdgeInAngularConstraint(e2StartOnPlane, e2EndOnPlane)
        
        iOldFlags[0] = true
    end
    
    return iOldFlags    
  end  
  
  def createObservers()
    self.add_observer(LDDDrDiEntityObserver.new)
      
    self.start.add_observer(LDDLabelsObserverForVertex.new)
    self.end.add_observer(LDDLabelsObserverForVertex.new)
  end

  def normalTheirSelfTwoEdges(iEdge, iFlags = [true, true, true], iMousePosition = Geom::Point3d.new(0,0,0))      
    face = self.common_face iEdge
    
    if face
      return face.normal
    else
      edge1Vector = self.vectorLDD(iFlags[1])
      edge2Vector = iEdge.vectorLDD(iFlags[2])
          
      normal = edge1Vector * edge2Vector          
      return normal if (normal.length > 0)
      
      point = self.getIntersectionPoint(iEdge)
      
      if point
        normal = edge1Vector * (point.vector_to iMousePosition)      
        return normal if (normal.length > 0)
      end
      
      normal = edge1Vector * Geom::Vector3d.new(1,0,0)
      return normal if (normal.length > 0)      
      
      normal = edge1Vector * Geom::Vector3d.new(0,1,0)
      return normal if (normal.length > 0)      
      
    end
    
    return Geom::Vector3d.new(0,0,1)    
  end
  
  def getCorrectPlanarAngle(iEdge, iValue, iFlags = [true, true, true])
      
    counter = 0
    
    if (not iFlags[1])
      counter += 1
    end
    
    if (not iFlags[2])
      counter += 1
    end     
      
    if(iValue <= Math::PI)
      if(counter == 1)
        return (iValue + Math::PI)
      else
        return iValue
      end
    else
      if(counter == 1)
        return (iValue - Math::PI)
      else
        return iValue
      end
    end    
    return iValue
  end
  
  def getCorrectAngle(iEdge, iValue, iFlags = [true, true, true])
      
    counter = 0
    
    if (not iFlags[1])
      counter += 1
    end
    
    if (not iFlags[2])
      counter += 1
    end     
    
    if(counter == 1)
      return (Math::PI - iValue)
    else
      return iValue
    end
  
    return iValue
  end
  
  def vectorLDD(iFlag)
    vector = self.line[1]    
    vector.reverse! if (not iFlag)
    return vector    
  end
  
  def angleToEdge(iEdge2, iFlags = [true, true, true])
    
    point0 = self.getIntersectionPoint(iEdge2)
    
    edge1Vector = self.vectorLDD(iFlags[1])
    edge2Vector = iEdge2.vectorLDD(iFlags[2])
    
    if point0
      face = self.common_face iEdge2
      
      if face      
        normal = face.normal
        angle = edge1Vector.getPlanarAngleForVector(edge2Vector, normal)
      else        
        angle = edge1Vector.angle_between edge2Vector
      end
    else
      angle = edge1Vector.angle_between edge2Vector      
    end
    
    if iFlags[0]
      return angle
    else
      return ((Math::PI)*2 - angle)
    end    
  end
  
  def commonVertex(iEdge) 
    vertex = self.start    
  
    if(vertex.used_by? iEdge)
      return vertex
    else    
      vertex = self.end
      
      if(vertex.used_by? iEdge)
        return vertex
      else    
        return nil
      end
    end    
    return nil
  end
  
  def midPoint
    return Geom.linear_combination 0.5, self.start.position, 0.5, self.end.position
  end  
end

#====================================================================================

#====================================================================================
class String
  def to_f_LDD
    actual_float =[]  
  
    if(self.to_f == 0.0)
    #warning! strange string
    
      if(self[0,1] == "~")
        srt_length = self.length
      
        if(self[1,srt_length-1].to_f == 0.0)      
          if self[1,1] == "0"
            return 0.0
          else
            return false
          end
        else
          #good
          #next step
          actual_float[0] = self[1,srt_length-1].to_f
        end    
      
      else        
        if self[0,1] == "0"
          return 0.0
        else
          return false
        end
      end    
    else
      actual_float[0] = self.to_f
    end
    
    value = actual_float[0] #.to_l
    
    if(value < 0.0)
      return false
    end
    
    return value 
  end
  
  def to_l_LDD
      
    if(self.to_f == 0.0)
    #warning! strange string
    
      if(self[0,1] == "~")
        srt_length = self.length
      
        if(self[1,srt_length-1].to_f == 0.0)      
          if self[1,1] == "0"
            return (0.0).to_l
          else
            return false
          end
        else
          #good
          #next step          
        end    
      
      else        
        if self[0,1] == "0"
          return (0.0).to_l
        else
          return false
        end
      end
    end
    
    value = self.to_l
    
    if(value.to_f < 0.0)
      return false
    end
    
    return value 
  end
end

#====================================================================================

class Sketchup::Vertex
  def createObservers
    self.add_observer(LDDDrDiEntityObserver.new)
      
    self.add_observer(LDDLabelsObserverForVertex.new)
  end
end

#====================================================================================
class Array
  def line_between?(iLine1, iLine2, iNormal)
    
  end
  def uniq_LDD    
    hash = Hash.new
    pointsArray = []
    
    self.each{ |vertex|
      position = vertex.position
      hash[position] = vertex
      pointsArray.push position
    }
    
    ary = []
        
    pointsArray.each{ |point|
      @actual = true
      ary.each{ |pp|
        if (pp == point)
          @actual = false 
          break
        end
      }
      (ary.push point) if @actual
    }
    
    if ary.length > 2    
            
      line = [ary[0], ary[1]]
      
      @actual = false
      
      ary.each{ |point|
        if not (point.on_line? line)
          @actual = true
          break          
        end
      }
      
      return nil if not @actual
      
      to_return = []      
      ary.each{ |point|
        to_return.push hash[point]
      }
      
      return to_return
    else
      return nil
    end
  end
end
#====================================================================================


file_loaded("ldd_sketchup_classes.rb")
