# Copyright 2008, LEDAS

# NO PART OF THIS SOFTWARE MAY BE RE-USED FOR ANY OTHER PURPOSE THAN TO RUN
# DRIVING DIMENSIONS WITHOUT THE EXPRESS AND WRITTEN APPROVAL OF LEDAS.


# YOU UNDERSTAND AND AGREE THAT LEDAS IS MAKING NO WARRANTY THAT THE SOFTWARE WILL
# MEET YOUR REQUIREMENTS, THAT THE SOFTWARE WILL BE ERROR OR BUG-FREE,
# REGARDING THE SECURITY, RELIABILITY OR PERFORMANCE OF THE SOFTWARE, OR THAT ANY
# ERRORS IN THE SOFTWARE WILL BE CORRECTED.  YOUR USE OF THE SOFTWARE IS AT YOUR SOLE RISK.
# THE SOFTWARE IS PROVIDED ON AN "AS IS" BASIS AND WITHOUT WARRANTY OF ANY KIND.
# TO THE EXTENT PERMITTED BY LAW, LEDAS DISCLAIMS ALL WARRANTIES AND CONDITIONS OF ANY KIND,
# WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES AND CONDITIONS
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT. ANY CONTENT 
# OR MATERIAL DOWNLOADED OR OTHERWISE OBTAINED THROUGH THE USE OF THE SOTWARE IS DONE AT
# YOUR OWN DISCRETION AND RISK AND YOU WILL BE SOLELY RESPONSIBLE FOR ANY DAMAGE TO YOUR
# COMPUTER SYSTEM OR OTHER DEVICE OR LOSS OF DATA THAT RESULTS FROM THE DOWNLOAD OF ANY
# SUCH CONTENT OR MATERIAL.  YOU CAN RECOVER FROM LEDAS AND ITS SUPPLIERS ONLY DIRECT DAMAGES
# UP TO THE AMOUNT YOU PAID FOR THE SOFTWARE.  YOU CANNOT RECOVER ANY OTHER DAMAGES, INCLUDING
# CONSEQUENTIAL, LOST PROFITS, SPECIAL, INDIRECT OR INCIDENTAL DAMAGES.

 
require 'sketchup.rb'


require "ldd\\ldd_class.rb"
require "ldd\\ldd_constraints.rb"
require "ldd\\ldd_common_functions.rb"


module LDD

  def textEntityToSelection(iView, iText, iSelection)
  
    dr_di_id = iText.get_attribute "LDDGUIAttributes", "Driving_Dimension", 0
            
    if(dr_di_id > 0)
    
      constraint = LDDConstraint.constraints.findConstraintOnID(dr_di_id)
      if constraint          
        iSelection.add(iText)
                              
        ent1 = constraint.entity1
        
        if (ent1.kind_of? Sketchup::Edge)            
          iSelection.add ent1
        elsif (ent1.kind_of? Sketchup::Curve)
          iSelection.add ent1.edges          
        end
        
        iSelection.add(constraint.entity2) if (constraint.entity2 && (not (constraint.entity2.kind_of? Sketchup::Vertex)))
      
        iView.tooltip = constraint.name
        iView.invalidate
      
        return constraint
      end            
    end
    
    return nil  
  end

  def ldd_pick(view, x, y)
    
    pickhelper = view.pick_helper x, y
    pickhelper.do_pick x,y	
    picked_entity = pickhelper.best_picked
      
    if(picked_entity == nil)
      return nil
    end
    
    if(picked_entity.kind_of? Sketchup::Edge)
    
      curve = picked_entity.curve
        
      if(curve == nil)
        
        if((picked_entity.smooth?)&&(picked_entity.soft?)) #on cylinder edge
          return nil
        else #not on cylinder edge
        
          inputpoint = view.inputpoint x, y 
          inputpoint.pick view, x, y 
          vertex = inputpoint.vertex 
        
          if (vertex)
            return vertex              
          else
            return picked_entity              
          end
          
        end #not on cylinder edge
        
      else #(curve == nil) - not true
        if(curve.kind_of? Sketchup::ArcCurve)
          return curve
        else
          eturn nil
        end
      end
    end      
    
    if(picked_entity.kind_of? Sketchup::Text)      
      return picked_entity
    end
    
    return nil
  end

  class Driving_Dimensions_Tool
    #-----------------------------------------------------------------------------------------------        
    @@ldd_tool_choosen = -1
    
    #-----------------------------------------------------------------------------------------------
    
    def Driving_Dimensions_Tool.active?
      return @@ldd_tool_choosen
    end
    
    #-----------------------------------------------------------------------------------------------
    
    def activate
      @@ldd_tool_choosen = 1
      #puts "activate"
    end
    
    def deactivate view
      @@ldd_tool_choosen = -1
      #puts "deactivate"
      self.reset
    end
    
    def suspend view 
      #puts "suspend"
      @@ldd_tool_choosen = 0
    end
    
    def resume view 
      #puts "resume"
      @@ldd_tool_choosen = 1
    end
    
    def initialize      
      @ip = Sketchup::InputPoint.new
      @ip2 = Sketchup::InputPoint.new
      @operatedConstraint = nil
      
      @picked_entity = nil
      
      mod = Sketchup.active_model
      mode = mod.get_attribute "LDDAttributes", "mode", nil
      if (not mode)
        mod.set_attribute "LDDAttributes", "mode", 52
      end	    		
    end
    #-----------------------------------------------------------------------------------------------
   
    #-----------------------------------------------------------------------------------------------
    def draw(view)
      @ip.draw(view)
    end
    #-----------------------------------------------------------------------------------------------	
       
    #-----------------------------------------------------------------------------------------------
    def onRButtonDown(flags, x, y, view)  
    end
    #-----------------------------------------------------------------------------------------------
    def onLButtonUp(flags, x, y, view)
      @ip.pick view, x, y
      view.invalidate
      
      if @operatedConstraint      
        if @operatedConstraint.kind_of? LDDConstraint 
           @operatedConstraint = nil          
           Sketchup.active_model.commit_operation_LDD
           recalculateLabelsPositionsTwoTimes
          return
        end
        
        if @operatedConstraint.kind_of? LDDPreConstraint          
          if((@ip.position.distance @ip2.position) > (1.0).to_l )
            @operatedConstraint = @operatedConstraint.nextStep(@ip.position, nil)
          end
          return
        end        
      end
      
    end
    #-----------------------------------------------------------------------------------------------
            
    #-----------------------------------------------------------------------------------------------
    
    def onLButtonDoubleClick(flags, x, y, view)
    
      selection = selection_DD(view, x, y,nil)
      if (selection)		  
        
        if (selection.kind_of? LDDConstraint) #Change constraint
          Sketchup.active_model.start_operation_LDD	"Change constraint"                     

          old_value = selection.value
            
          return if (not selection.change!)
                                          
          result = ldd_solve(nil)	 
            
          if (result == 0)
            Sketchup.active_model.commit_operation_LDD
            recalculateLabelsPositionsTwoTimes if LDDLabelsObserverForVertex.changed?
          else            
            Sketchup.active_model.abort_operation_LDD
            selection.change!(old_value)#([old_value.to_s])            
            LDDConstraint.constraints.recalculateLabelsPositions                      
            return
          end
        end
      end
    end
    #-----------------------------------------------------------------------------------------------
    def onLButtonDown(flags, x, y, view)
      
      @ip2.pick view, x, y   
      #pickhelper = view.pick_helper x, y
      #pickhelper.do_pick x,y
            
      if @operatedConstraint
      
        if @operatedConstraint.kind_of? LDDPreConstraint
          @operatedConstraint = @operatedConstraint.nextStep(@ip2.position, selection_DD(view, x, y, @operatedConstraint))
        else
          #ERROR
        end
      
      else # not @operatedConstraint
        selection = selection_DD(view, x, y, @operatedConstraint)
        
        if selection
          if selection.kind_of? LDDConstraint #Move label           
            Sketchup.active_model.start_operation_LDD "Move Driving Dimensions"
            @operatedConstraint = selection
            return
          end
          
          if selection.kind_of? Sketchup::Entity
            Sketchup.active_model.start_operation_LDD "Driving Dimensions"
            @operatedConstraint = LDDPreConstraint.new(selection,@ip2.position)
          end
          
        end      
      end      
    end
    #-----------------------------------------------------------------------------------------------
    
    #-----------------------------------------------------------------------------------------------
    def onCancel(flag, view)
        view.invalidate 
        self.reset
    end
    #-----------------------------------------------------------------------------------------------
    def reset          
      @operatedConstraint = nil      
      
      Sketchup.active_model.abort_operation_LDD	
    end
    #-----------------------------------------------------------------------------------------------
    def onMouseMove (flags, x, y, view)
      @ip.pick view, x, y
      view.invalidate
      
      if @operatedConstraint
      
        if @operatedConstraint.kind_of? LDDConstraint                    
          @operatedConstraint.drivingDimensionEntity.moveWithMousePosition(@operatedConstraint.type, 
                                                                            @ip.position,
                                                                            @operatedConstraint.entity1,
                                                                            @operatedConstraint.entity2)
          return
        end
          	      
        if @operatedConstraint.kind_of? LDDPreConstraint          
          @operatedConstraint.setCurrentPosition(@ip.position, selection_DD(view, x, y, @operatedConstraint))
          return
        end
      else
        selection_DD(view, x, y, @operatedConstraint)      
      end
      
    end
    
    #-----------------------------------------------------------------------------------------------
        
    #-----------------------------------------------------------------------------------------------
    
    def selection_DD(view, x, y, iLDDPreConstraint)
          
      message = ""
      
      sss = Sketchup.active_model.selection
      sss.clear
            
      entity = ldd_pick(view, x, y)
      
      return nil if not entity
      
      if (not iLDDPreConstraint)        
        if entity.kind_of? Sketchup::Text        
          return textEntityToSelection(view, entity, sss)
        else
          type = getConstraintType(entity)        
        end
      else
        return nil if((not iLDDPreConstraint.entity1)||(iLDDPreConstraint.entity1.deleted?))
        type = getConstraintType(iLDDPreConstraint.entity1, entity)
      end
      
      return nil if (type == 0)
      
      case type      
        when LDD_constraints_types::VERTEX_SOMETHING
          message << "Vertex"
          
        when LDD_constraints_types::EDGE_LENGTH
          message << "Edge"
          sss.add entity
          
        when LDD_constraints_types::ARCCURVE_RADIUS
          message << "ArcCurve"
          sss.add entity.edges
          
        when LDD_constraints_types::EDGE_EDGE_DISTANCE
          message << "Distance to Edge"
          sss.add entity
          sss.add iLDDPreConstraint.entity1
          
        when LDD_constraints_types::VERTEX_EDGE_DISTANCE                    
            message << "Distance to Edge"
            sss.add entity                      
          
        when LDD_constraints_types::EDGE_VERTEX_DISTANCE          
            message << "Distance to Vertex"            
            sss.add iLDDPreConstraint.entity1
                  
        when LDD_constraints_types::VERTEX_VERTEX_DISTANCE
          message << "Distanse to Vertex"
          
        when LDD_constraints_types::EDGE_EDGE_PLANAR_ANGLE
          message << "Angle to Edge"
          sss.add entity
          sss.add iLDDPreConstraint.entity1                
          
        when LDD_constraints_types::EDGE_EDGE_ANGLE
          message << "Angle to Edge"
          sss.add entity
          sss.add iLDDPreConstraint.entity1                
      end
      
      view.tooltip = message
      view.invalidate
    
     return entity
      
    end #selection_DD
  end
end
#============================================================================================

file_loaded("ldd_tool.rb")