# Matt666 Sept. 2008

# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

#Draw some points
#V1.0

class PointTool
	def initialize
		@ip1 = nil
		@ip2 = nil
		@xdown = 0
		@ydown = 0
	end

	def activate
		Sketchup.active_model.start_operation "Point"
		@ip = Sketchup::InputPoint.new
		@ip1 = Sketchup::InputPoint.new
		@ip2 = Sketchup::InputPoint.new
		@drawn = false
		if Sketchup.active_model.rendering_options["HideConstructionGeometry"] == true 
			Sketchup.active_model.rendering_options["HideConstructionGeometry"] = false 
		end
		self.reset(nil)
	end
  
	def deactivate(view)
		view.invalidate if @drawn
	end
	
	def onCancel(flag, view)
		Sketchup.active_model.commit_operation
		Sketchup.send_action "selectSelectionTool:"
	end
  
	 def reset(view)
		@state = 0
		Sketchup::set_status_text ("Specify first Point:"), SB_PROMPT
		@ip.clear
		@ip1.clear
		@ip2.clear
		if view
			view.tooltip = nil
			view.invalidate if @drawn
		end
		@drawn = false
		@dragging = false
	end
	
	def onMouseMove(flags, x, y, view)
		case @state
		
			when 0
			@ip.pick view, x, y
			if @ip != @ip1
				view.invalidate if @ip.display? or @ip1.display?
				@ip1.copy! @ip
				view.tooltip = @ip1.tooltip
			end
			view.tooltip = "First point:"
		
			when 1,2
			@ip2.pick view, x, y, @ip1
			view.tooltip = @ip2.tooltip if( @ip2.valid? )
			view.invalidate
			if @ip2.valid?
				length = @ip1.position.distance(@ip2.position)
				Sketchup::set_status_text length.to_s, SB_VCB_VALUE
			end
			view.tooltip = "Next point:"
		end
	end

	def onLButtonDown(flags, x, y, view)
		case @state
			when 0
			@ip1.pick view, x, y
			if @ip1.valid?
				@state = 1
				Sketchup::set_status_text ("Click for next point:"), SB_PROMPT
				Sketchup::set_status_text ("Length"), SB_VCB_LABEL
				@xdown = x
				@ydown = y
			end
			
			when 1,2
			if @ip2.valid? && @ip2 != @ip1
				self.create_geometry(@ip1.position,@ip2.position,view)
				@state = 2
				@ip1.copy! @ip2
			end			
		end
		view.lock_inference
	end

	def onLButtonUp(flags, x, y, view)
		if @dragging && @ip2.valid?
			self.create_geometry(@ip1.position, @ip2.position,view)
			self.reset(view)
		end
	end

	def onKeyDown(key, repeat, flags, view)
		if key == CONSTRAIN_MODIFIER_KEY && repeat == 1
			@shift_down_time = Time.now
			if( view.inference_locked? )
				view.lock_inference
			elsif @state == 0 && @ip1.valid?
				view.lock_inference @ip1
				view.line_width = 3
			elsif @state != 0 && @ip2.valid?
				view.lock_inference @ip2, @ip1
				view.line_width = 3
			end
		end
	end

	def onKeyUp(key, repeat, flags, view)
		view.lock_inference if key == CONSTRAIN_MODIFIER_KEY && view.inference_locked? && (Time.now - @shift_down_time) > 0.5
	end

	def onUserText(text, view)
		return if @state == 0
		return if not @ip2.valid?
		begin
			value = text.to_l
		rescue
			puts "Cannot convert #{text} to a Length"
			value = nil
			Sketchup::set_status_text "", SB_VCB_VALUE
		end
		return if !value
		pt1 = @ip1.position
		vec = @ip2.position - pt1
		return if vec.length == 0.0
		vec.length = value
		pt2 = pt1 + vec
		self.create_geometry(pt1, pt2, view)
		@state = 2
		pt2 = view.screen_coords pt2
		@ip1.pick view,pt2[0],pt2[1]
	end
  
	def draw(view)
	    if @ip1.valid?
			if @ip1.display?
				@ip1.draw(view)
				@drawn = true
			end
			if @ip2.valid?
				@ip2.draw(view) if @ip2.display?
				view.set_color_from_line(@ip1, @ip2)
				self.draw_geometry(@ip1.position, @ip2.position, view)
				@drawn = true
			end
		end
	end

	def create_geometry(p1, p2, view)
		view.model.active_entities.add_cpoint(p1) if @state != 2
		view.model.active_entities.add_cpoint(p2)
		view.model.active_entities.add_cline(p1,p2)
	end

	def draw_geometry(pt1, pt2, view)
		view.draw_line(pt1, pt2)
	end
	
	def PointTool.tool
		Sketchup.active_model.select_tool PointTool.new
	end
end

if not file_loaded?("point.rb")
   UI.menu("Tools").add_item("Construction Geometry") {PointTool.tool}
end
file_loaded("xGeometry.rb")